from __future__ import print_function
__all__ = ['uamiv']
__doc__ = """
.. _Read
:mod:`Read` -- uamiv Read interface
============================================

.. module:: Read
   :platform: Unix, Windows
   :synopsis: Provides :ref:`PseudoNetCDF` random access read for CAMx
              uamiv files.  See PseudoNetCDF.sci_var.PseudoNetCDFFile
              for interface details
.. moduleauthor:: Barron Henderson <barronh@unc.edu>
"""

# Distribution packages
import unittest
import struct

# Site-Packages
from numpy import zeros, array, newaxis

# This Package modules
from PseudoNetCDF.camxfiles.timetuple import timediff, timerange
from PseudoNetCDF.camxfiles.util import sliceit
from PseudoNetCDF.camxfiles.units import get_uamiv_units, get_chemparam_names
from PseudoNetCDF.camxfiles.FortranFileUtil import OpenRecordFile, read_into
from PseudoNetCDF.camxfiles.FortranFileUtil import Int2Asc
from PseudoNetCDF.sci_var import PseudoNetCDFFile, PseudoNetCDFVariables


# for use in identifying uncaught nan
listnan = struct.unpack('>f', b'\xff\xc0\x00\x00')[0]
checkarray = zeros((1,), 'f')
checkarray[0] = listnan
array_nan = checkarray[0]


class uamiv(PseudoNetCDFFile):
    """
    uamiv provides a PseudoNetCDF interface for CAMx
    uamiv files.  Where possible, the inteface follows
    IOAPI conventions (see www.baronams.com).

    ex:
        >>> uamiv_path = 'camx_uamiv.bin'
        >>> rows,cols = 65,83
        >>> uamivfile = uamiv(uamiv_path,rows,cols)
        >>> uamivfile.variables.keys()
        ['TFLAG', 'O3', 'NO', 'NO2', ...]
        >>> tflag = uamivfile.variables['TFLAG']
        >>> tflag.dimensions
        ('TSTEP', 'VAR', 'DATE-TIME')
        >>> tflag[0,0,:]
        array([2005185,       0])
        >>> tflag[-1,0,:]
        array([2005185,  240000])
        >>> v = uamivfile.variables['O3']
        >>> v.dimensions
        ('TSTEP', 'LAY', 'ROW', 'COL')
        >>> v.shape
        (25, 28, 65, 83)
        >>> uamivfile.dimensions
        {'TSTEP': 25, 'LAY': 28, 'ROW': 65, 'COL': 83}
    """

    emiss_hdr_fmt = "10i60i3ifif"
    grid_hdr_fmt = "ffiffffiiiiifff"
    cell_hdr_fmt = "iiii"
    time_hdr_fmt = "ifif"
    spc_fmt = "10i"
    id_fmt = "i" + spc_fmt
    id_size = struct.calcsize(id_fmt)
    data_fmt = "f"
    ione = 1
    idum = 0
    rdum = 0.

    def __init__(self, rf, chemparam=None):
        """
        Initialization included reading the header and learning
        about the format.

        see __readheader and __gettimestep() for more info
        """

        self.rffile = OpenRecordFile(rf)
        if chemparam is None:
            self._aerosol_names = None
        else:
            self._aerosol_names = get_chemparam_names(chemparam)

        self.padded_time_hdr_size = struct.calcsize(self.time_hdr_fmt + "ii")
        self.__readheader()
        self.__gettimestep()
        self.dimensions = {}
        self.createDimension('LAY', self.nlayers)
        self.createDimension('COL', self.nx)
        self.createDimension('ROW', self.ny)
        self.createDimension('TSTEP', self.time_step_count)
        self.createDimension('DATE-TIME', 2)

        self.variables = PseudoNetCDFVariables(
            self.__var_get, [sn.strip() for sn in self.spcnames])

    def __var_get(self, key):
        units = get_uamiv_units(self.name, key, self._aerosol_names)
        spcnames = [sn.strip() for sn in self.spcnames]
        if self.name == 'EMISSIONS ':
            def constr(spc):
                return self.getArray(
                    nspec=spcnames.index(spc)).squeeze()[:, newaxis, :, :]

            def decor(spc):
                return dict(units=units, var_desc=spc,
                            long_name=spc.ljust(16))
        else:
            ntimes = len(self.dimensions['TSTEP'])
            nlays = len(self.dimensions['LAY'])
            nrows = len(self.dimensions['ROW'])
            ncols = len(self.dimensions['COL'])

            def constr(spc):
                return self.getArray(nspec=spcnames.index(
                    spc)).squeeze().reshape(ntimes, nlays, nrows, ncols)

            def decor(spc):
                return dict(units=units, var_desc=spc.ljust(16),
                            long_name=spc.ljust(16))

        values = constr(key)
        var = self.createVariable(key, 'f', ('TSTEP', 'LAY', 'ROW', 'COL'))
        var[:] = values
        for k, v in decor(key).items():
            setattr(var, k, v)
        return var

    def header(self):
        rdum = self.rdum
        idum = self.idum
        ione = self.ione
        return [
            [self.name, self.note, ione, self.nspec, self.start_date,
                self.start_time, self.end_date, self.end_time],
            [rdum, rdum, self.iutm, self.xorg, self.yorg, self.delx, self.dely,
                self.nx, self.ny, self.nz, idum, idum, rdum, rdum, rdum],
            [ione, ione, self.nx, self.ny],
            self.spcnames
        ]

    def __readheader(self):
        """
        __readheader reads the header section of the ipr file
        it initializes each header field (see CAMx Users Manual for a list)
        as properties of the ipr class
        """
        vals = self.rffile.read(self.emiss_hdr_fmt)
        self.name = vals[0:10]
        self.note = vals[10:70]
        ione = vals[70]
        self.nspec = vals[71]
        self.start_date, self.start_time = vals[72], vals[73]
        self.end_date, self.end_time = vals[74], vals[75]

        self.name = Int2Asc(self.name)
        self.note = Int2Asc(self.note)

        vals = self.rffile.read(self.grid_hdr_fmt)
        self.rdum, rdum, self.iutm = vals[0:3]
        self.xorg, self.yorg, self.delx, self.dely = vals[3:7]
        self.nx, self.ny, self.nz = vals[7:10]
        idum, self.idum, rdum, rdum, rdum = vals[10:]

        if self.name == 'EMISSIONS ':
            # Special case of gridded emissions
            # Seems to be same as avrg
            self.nlayers = 1
        else:
            self.nlayers = self.nz
        self.ione, ione, nx, ny = self.rffile.read(self.cell_hdr_fmt)
        if not (self.nx, self.ny) == (nx, ny):
            raise ValueError(("nx, ny defined first as %i, %i and then " +
                              "as %i, %i") % (self.nx, self.ny, nx, ny))
        species_temp = self.rffile.read(self.nspec * self.spc_fmt)
        self.spcnames = []
        for i in range(0, self.nspec * 10, 10):
            self.spcnames.append(Int2Asc(species_temp[i:i + 10]))

        self.data_start_byte = self.rffile.record_start
        start_date, start_time, end_date, end_time = self.rffile.read(
            self.time_hdr_fmt)

        self.time_step = timediff(
            (start_date, start_time), (end_date, end_time))
        mystep = (2400, 24)[int(self.time_step % 2)]
        self.time_step_count = int(timediff((self.start_date, self.start_time),
                                            (self.end_date, self.end_time),
                                            mystep) // self.time_step)
        if self.name == 'AIRQUALITY':
            self.time_step_count = 1
            self.start_date = self.end_date
        self.record_size = self.rffile.record_size
        self.padded_size = self.record_size + 8
        self.cell_count = (self.record_size - struct.calcsize("i10i")
                           ) // struct.calcsize(self.data_fmt)
        self.record_fmt = ("i10i") + self.data_fmt * (self.cell_count)

    def __gettimestep(self):
        """
        this is taken care of in the readheader routine
        record format provides start and end for each hour,
        which translates to t1 and t2
        """
        pass

    def __timerecords(self, dt):
        """
        Calculate the number of records to increment to reach time (d,t)
        """
        d, t = dt
        nsteps = int(
            timediff((self.start_date, self.start_time), (d, t)) /
            self.time_step)
        nspec = self.__spcrecords(self.nspec + 1)
        return nsteps * nspec

    def __layerrecords(self, k):
        """Calculate the number of records to increment to reach layer k
        """
        return k - 1

    def __spcrecords(self, spc):
        """
        Calculated number of records before spc
        """
        return (spc - 1) * self.__layerrecords(self.nlayers + 1)

    def __recordposition(self, date, time, spc, k):
        """
        Use time (d,t), spc, and k to calculate number of records before
        desired record

        date - integer julian
        time - float
        spc - integer
        k - integer
        """
        ntime = self.__timerecords((date, time))
        nk = self.__layerrecords(k)
        nid = ntime // self.nspec // self.nlayers
        nspec = 0
        if spc != 0:
            nid += 1
            nspec = self.__spcrecords(spc)

        out = (self.data_start_byte + (nspec + nk + ntime) *
               self.padded_size + nid * self.padded_time_hdr_size)
        return out

    def seek(self, date=None, time=None, spc=-1, k=0, chkvar=True):
        """
        Move file cursor to the beginning of the specified record
        see __recordposition for parameter definitions
        """
        spc += 1
        if date is None:
            date = self.start_date
        if time is None:
            time = self.start_time

        if (
            chkvar and
            timediff((self.end_date, self.end_time), (date, time), 24) > 0 or
            timediff((self.start_date, self.start_time), (date, time), 24) < 0
        ):
            raise KeyError(("Gridded emission file includes (%i,%6.1f) " +
                            "thru (%i,%6.1f); you requested (%i,%6.1f)") %
                           (self.start_date, self.start_time,
                            self.end_date, self.end_time, date, time))
        if chkvar and spc < 1 or spc > self.nspec:
            raise KeyError(("Gridded emission file include species 1 thru " +
                            "%i; you requested %i") % (self.nspec, spc))

        # self.rffile._newrecord(self.__recordposition(date,time,1,0))
        # start_date,start_time,end_date,end_time=self.rffile.read("ifif")
        self.rffile._newrecord(self.__recordposition(date, time, spc, k))

    def read(self):
        """
        Provide direct access to record file read
        """
        return self.rffile.read(self.record_fmt)

    def read_into(self, dest):
        """
        Transfer values from current record to dest
        dest - numeric or numpy array
        """
        return read_into(self.rffile, dest, self.id_fmt, self.data_fmt)

    def seekandreadinto(self, dest, date=None, time=None, spc=1, k=1):
        """
        see seek and read_into
        """
        self.seek(date, time, spc, k)
        self.read_into(dest)

    def seekandread(self, date=None, time=None, spc=1, k=1):
        """
        see seek and read
        """
        self.seek(date, time, spc, k)
        return self.read()

    def values(self):
        for d, t, spc, k in self.__iter__():
            yield self.seekandread(d, t, spc, k)

    def items(self):
        for d, t, spc, k in self.__iter__():
            yield d, t, spc, k, self.seekandread(d, t, spc, k)

    def keys(self):
        for d, t in self.timerange():
            for spc in range(len(self.spcnames)):
                for k in range(1, self.nlayers + 1):
                    yield d, t, spc, k
    __iter__ = keys

    def close(self):
        self.rffile.infile.close()

    def getArray(self, krange=slice(1, None), nspec=slice(None),
                 nx=slice(None), ny=slice(None)):
        """Method takes slice arguments. Alternatively, takes a hashable object
        with 2 values (e.g., the list: [0,3]).
        Arguments:
        krange    vertical slice (1 indexed)
        nspec     species  slice (0 indexed)
        nx        column   slice (0 indexed)
        ny        row      slice (0 indexed)
        """
        krange = sliceit(krange)
        nspec = sliceit(nspec)
        nx = sliceit(nx)
        ny = sliceit(ny)

        a = zeros(
            (
                self.time_step_count,
                len(range(*nspec.indices(self.nspec))),
                len(range(*krange.indices(self.nlayers + 1))),
                self.ny,
                self.nx), 'f')
        nlay = self.nlayers
        for ti, (d, t) in enumerate(self.timerange()):
            for sidx, spc in enumerate(range(*nspec.indices(self.nspec))):
                for kidx, k in enumerate(range(*krange.indices(nlay + 1))):
                    self.seekandreadinto(a[ti, sidx, kidx, ...], d, t, spc, k)

        return a[..., ny, nx]

    def timerange(self):
        return timerange((self.start_date, self.start_time),
                         (self.end_date, self.end_time), self.time_step, 24)


class TestuamivRead(unittest.TestCase):
    def runTest(self):
        pass

    def setUp(self):
        pass

    def testGE(self):
        import PseudoNetCDF.testcase
        emissfile = uamiv(PseudoNetCDF.testcase.camxfiles_paths['uamiv'])
        v = emissfile.variables['NO2']
        self.assertTrue((v == array(
            [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
             0.00000000e+00, 0.00000000e+00, 1.24175494e-04, 2.79196858e-04,
             1.01672206e-03, 4.36782313e-04, 0.00000000e+00, 1.54810550e-04,
             3.90250643e-04, 6.18023798e-04, 3.36963218e-04, 0.00000000e+00,
             1.85579920e-04, 1.96825975e-04, 2.16468165e-04, 2.19882189e-04],
            dtype='f').reshape(1, 1, 4, 5)).all())


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
__all__ = ['bcon_profile', 'icon_profile']
from ._ioapi import ioapi_base
try:
    from StringIO import StringIO as BytesIO
except ImportError:
    from io import BytesIO
import numpy as np
# from datetime import datetime


def _getunit(varkey):
    if varkey.startswith('A') and varkey[-1:] in ('I', 'J', 'K'):
        unit = 'micrograms/m**3'
    elif varkey.startswith('ASEA'):
        unit = 'micrograms/m**3'
    elif varkey.startswith('NUM'):
        unit = '#/m**3'
    elif varkey.startswith('SRF'):
        unit = 'm**2/m**3'
    else:
        unit = 'ppmV'
    return unit.ljust(16)


def _profile2dict(lines, fieldnames):
    import re

    data = np.recfromtxt(
        BytesIO(
            bytes(
                re.sub(r'[ ]+"', '"', '\n'.join(lines)).replace('"', ''),
                encoding='ascii'
            )
        ),
        delimiter=' ', names=fieldnames,
        converters=dict(names=lambda x: x.strip()),
    )
    return data


class icon_profile(ioapi_base):
    def __init__(self, path):
        lines = open(path).read().split('\n')
        header = lines[3].split()
        nlay, nspc = [int(_v) for _v in header[:2]]
        sigmas = [float(_v) for _v in header[2:]]
        nsigmas = len(sigmas)
        # try:
        #     dateo = datetime.strptime(lines[4].strip(), '%Y-%m-%d')
        # except Exception:
        #     date, time = [int(_v) for _v in lines[4].split()]
        starts = [5]
        ends = [s + nspc for s in starts]
        keys = ['all']
        fieldnames = ('name',) + tuple(['s%f' % i for i in sigmas])
        data = dict([
            (k, _profile2dict(lines[s:e], fieldnames))
            for k, s, e in zip(keys, starts, ends)
        ])

        profile_spcs = np.char.strip(data[keys[0]]['name'])
        data_type = data[keys[0]].dtype
        self.createDimension('sigma', nsigmas)
        self.createDimension('LAY', nsigmas - 1)
        self.createVariable('sigma', 'f', ('sigma',),
                            values=np.array(sigmas), units='sigma')
        self.createVariable('LAY', 'f', ('LAY',), values=np.array(
            sigmas).repeat(2, 0)[1:-1].reshape(-1, 2).mean(1), units='sigma')
        self.VGLVLS = self.variables['sigma']
        self.VGTOP = 5000
        ks = keys[1:]
        for k in ks:
            try:
                assert((np.char.strip(data[k]['name']) == profile_spcs).all())
                assert(data[k].dtype == data_type)
                assert(data[k].dtype == data_type)
            except AssertionError:
                raise IOError('File is corrupt or inconsistent')
        varlist = []
        for a in data['all']:
            varkey = a[0].strip().decode()
            self.createVariable(
                varkey, 'f', ('LAY',), units=_getunit(varkey),
                values=np.array([tuple(a)[1:]])[0].astype('f'),
                long_name=varkey.ljust(16), var_desc=varkey.ljust(16))
            varlist.append(varkey.ljust(16))
        self.NVARS = len(varlist)
        self.createDimension('VAR', self.NVARS)
        setattr(self, 'VAR-LIST', ''.join(varlist))

    def toGrid(self, ncols, nrows, nsteps=1):
        """
        Parameters
        ----------
        ncols  : number of ncols for output file
        nrows  : number of rows for output file
        nsteps : number of output time steps
        """
        out = self.subsetVariables(['sigma', 'LAY'], exclude=True)
        out = out.insertDimension(TSTEP=nsteps, before='LAY')
        out = out.insertDimension(ROW=nrows, after='LAY')
        out = out.insertDimension(COL=ncols, after='ROW')
        return out

    def toGriddedFile(self, ioapifile):
        """
        Parameters
        ----------
        ioapifile : file with IOAPI metadata
        """
        nsteps = len(ioapifile.dimensions['TSTEP'])
        nrows = len(ioapifile.dimensions['ROW'])
        ncols = len(ioapifile.dimensions['COL'])
        # interp to ioapi vglvls
        out = self.interpSigma(vglvls=ioapifile.VGLVLS, vgtop=ioapifile.VGTOP)
        # repeat to grid
        out = out.toGrid(ncols, nrows, nsteps)
        for pk in ioapifile.ncattrs():
            if pk in ('VAR-LIST', 'FILEDESC', 'HISTORY'):
                continue
            setattr(out, pk, getattr(ioapifile, pk))
        out.updatemeta()
        return out


class bcon_profile(ioapi_base):
    def __init__(self, path):
        lines = open(path).read().split('\n')
        header = lines[3].split()
        nlay, nspc = [int(_v) for _v in header[:2]]
        sigmas = [float(_v) for _v in header[2:]]
        nsigmas = len(sigmas)
        # try:
        #     dateo = datetime.strptime(lines[4].strip(), '%Y-%m-%d')
        # except Exception:
        #     date, time = [int(_v) for _v in lines[4].split()]
        starts = [5 + i + i * nspc for i in range(4)]
        ends = [s + 1 + nspc for s in starts]
        keys = [lines[s].strip().lower() for s in starts]
        fieldnames = ('name',) + tuple(['s%f' % i for i in sigmas])
        data = dict([
            (k, _profile2dict(lines[s + 1:e], fieldnames))
            for k, s, e in zip(keys, starts, ends)
        ])

        profile_spcs = np.char.strip(data[keys[0]]['name'])
        data_type = data[keys[0]].dtype
        self.createDimension('sigma', nsigmas)
        self.createDimension('LAY', nsigmas - 1)
        self.createDimension('south_east_north_west', 4)
        self.createVariable('sigma', 'f', ('sigma',),
                            values=np.array(sigmas), units='sigma')
        self.createVariable('LAY', 'f', ('LAY',), values=np.array(
            sigmas).repeat(2, 0)[1:-1].reshape(-1, 2).mean(1), units='sigma')
        ks = keys[1:]
        self.VGLVLS = self.variables['sigma']
        self.VGTOP = 5000
        for k in ks:
            try:
                assert((np.char.strip(data[k]['name']) == profile_spcs).all())
                assert(data[k].dtype == data_type)
                assert(data[k].dtype == data_type)
            except AssertionError:
                raise IOError('File is corrupt or inconsistent')
        varlist = []
        zwsen = zip(data['west'], data['south'], data['east'], data['north'])
        for w, s, e, n in zwsen:
            assert(w[0] == s[0] and e[0] == n[0] and n[0] == s[0])
            varkey = w[0].strip().decode()
            tmpvals = np.array(
                [(lambda x: tuple(x)[1:])(_v) for _v in [s, e, n, w]]
            ).T.astype('f')
            self.createVariable(
                varkey, 'f', ('LAY', 'south_east_north_west'),
                units=_getunit(varkey),
                values=tmpvals,
                long_name=varkey.ljust(16), var_desc=varkey.ljust(16))
            varlist.append(varkey.ljust(16))
        self.NVARS = len(varlist)
        self.createDimension('VAR', self.NVARS)
        setattr(self, 'VAR-LIST', ''.join(varlist))

    def toPerim(self, ncols, nrows, nsteps=1):
        outf = ioapi_base()
        nperim = nrows * 2 + ncols * 2 + 4
        outf.createDimension('TSTEP', nsteps)
        outf.createDimension('DATE-TIME', 2)
        outf.createDimension('LAY', len(self.dimensions['LAY']))
        outf.createDimension('VAR', len(self.dimensions['VAR']))
        outf.createDimension('PERIM', nperim)
        varlist = []
        for vk, vv in self.variables.items():
            if vk in ('sigma', 'LAY'):
                continue
            outv = outf.copyVariable(vv, key=vk, dtype='f', dimensions=(
                'TSTEP', 'LAY', 'PERIM'), withdata=False)
            start = 0
            end = ncols + 1
            outv[:, :, start:end] = vv[None, :, [0]]
            start = end
            end += nrows + 1
            outv[:, :, start:end] = vv[None, :, [1]]
            start = end
            end += ncols + 1
            outv[:, :, start:end] = vv[None, :, [2]]
            start = end
            end += nrows + 1
            outv[:, :, start:end] = vv[None, :, [3]]
            varlist.append(vk.ljust(16))
        outf.NVARS = len(varlist)
        setattr(outf, 'VAR-LIST', ''.join(varlist))
        outf.updatemeta()
        return outf

    def toPerimFile(self, ioapifile):
        """
        Parameters
        ----------
        ioapifile : file with IOAPI metadata
        """
        nsteps = len(ioapifile.dimensions['TSTEP'])
        try:
            nrows = len(ioapifile.dimensions['ROW'])
            ncols = len(ioapifile.dimensions['COL'])
        except Exception:
            nrows = ioapifile.NROWS
            ncols = ioapifile.NCOLS
        # interp to ioapi vglvls
        out = self.interpSigma(vglvls=ioapifile.VGLVLS, vgtop=ioapifile.VGTOP)
        # repeat to grid
        out = out.toPerim(ncols, nrows, nsteps)
        for pk in ioapifile.ncattrs():
            if pk in ('VAR-LIST', 'FILEDESC', 'HISTORY'):
                continue
            setattr(out, pk, getattr(ioapifile, pk))
        out.updatemeta()
        return out


if __name__ == '__main__':
    po = bcon_profile('testdata/profile.dat')
    from PseudoNetCDF.pncdump import pncdump
    pncdump(po)
    print(po.variables['ATOL1J'].shape)
    print(po.variables['ATOL1J'].dimensions)



__all__ = ['uamiv']
__doc__ = """
.. _Memmap
:mod:`Memmap` -- uamiv Memmap interface
============================================

.. module:: Memmap
   :platform: Unix, Windows
   :synopsis: Provides :ref:`PseudoNetCDF` memory map for CAMx
              uamiv files.  See PseudoNetCDF.sci_var.PseudoNetCDFFile
              for interface details
.. moduleauthor:: Barron Henderson <barronh@unc.edu>
"""

# Distribution packages
import unittest
# from warnings import warn

# Site-Packages
from numpy import array, memmap, dtype, linspace

# This Package modules
from PseudoNetCDF.sci_var import PseudoIOAPIVariable
from PseudoNetCDF.sci_var import PseudoNetCDFVariables
from PseudoNetCDF.ArrayTransforms import ConvertCAMxTime
from PseudoNetCDF.camxfiles.units import get_uamiv_units, get_chemparam_names
from PseudoNetCDF.cmaqfiles import ioapi_base
# from PseudoNetCDF.sci_var import PseudoNetCDFFile
# from PseudoNetCDF.conventions.ioapi import add_cf_from_ioapi
# for use in identifying uncaught nan


class uamiv(ioapi_base):
    """
    uamiv provides a PseudoNetCDF interface for CAMx
    uamiv files.  Where possible, the inteface follows
    IOAPI conventions (see www.baronams.com).

    ex:
        >>> uamiv_path = 'camx_uamiv.bin'
        >>> uamivfile = uamiv(uamiv_path)
        >>> uamivfile.variables.keys()
        ['TFLAG', 'O3', 'NO', 'NO2', ...]
        >>> tflag = uamivfile.variables['TFLAG']
        >>> tflag.dimensions
        ('TSTEP', 'VAR', 'DATE-TIME')
        >>> tflag[0,0,:]
        array([2005185,       0])
        >>> tflag[-1,0,:]
        array([2005185,  240000])
        >>> v = uamivfile.variables['O3']
        >>> v.dimensions
        ('TSTEP', 'LAY', 'ROW', 'COL')
        >>> v.shape
        (25, 28, 65, 83)
        >>> uamivfile.dimensions
        {'TSTEP': 25, 'LAY': 28, 'ROW': 65, 'COL': 83}
    """

    __ione = 1
    __idum = 0
    __rdum = 0.

    def _make_header_fmt(self, ep=None):
        if ep is None:
            ep = self.__endianprefix
        self.__emiss_hdr_fmt = dtype(dict(
            names=['SPAD', 'name', 'note', 'itzon', 'nspec', 'ibdate', 'btime',
                   'iedate', 'etime', 'EPAD'],
            formats=['i', '(10, 4)S1', '(60,4)S1', 'i', 'i', 'i', 'f', 'i',
                     'f', 'i'])).newbyteorder(ep)
        self.__grid_hdr_fmt = dtype(dict(
            names=['SPAD', 'plon', 'plat', 'iutm', 'xorg', 'yorg', 'delx',
                   'dely', 'nx', 'ny', 'nz', 'iproj', 'istag', 'tlat1',
                   'tlat2', 'rdum', 'EPAD'],
            formats=['i', 'f', 'f', 'i', 'f', 'f', 'f', 'f', 'i', 'i', 'i',
                     'i', 'i', 'f', 'f', 'f', 'i'])).newbyteorder(ep)
        self.__cell_hdr_fmt = dtype(dict(
            names=['SPAD', 'ione1', 'ione2', 'nx', 'ny', 'EPAD'],
            formats=['i', 'i', 'i', 'i', 'i', 'i'])).newbyteorder(ep)
        self.__time_hdr_fmt = dtype(dict(
            names=['SPAD', 'ibdate', 'btime', 'iedate', 'etime', 'EPAD'],
            formats=['i', 'i', 'f', 'i', 'f', 'i'])).newbyteorder(ep)
        self.__spc_fmt = dtype("(10,4)S1").newbyteorder(ep)

    def __init__(self, rf, mode='r', P_ALP=None, P_BET=None, P_GAM=None,
                 XCENT=None, YCENT=None, GDTYP=None, endian='big',
                 chemparam=None):
        """
        Parameters
        ----------
        rf : string or RecordFile
            usually a path to a CAMx formatted file, can be a FortranFileUtil
            RecordFile object.
        mode : string
            file open mode read ('r'), write ('w'), append ('a', 'r+')
        P_ALP : float
            see IOAPI GRIDDESC documentation
        P_BET : float
            see IOAPI GRIDDESC documentation
        P_GAM : float
            see IOAPI GRIDDESC documentation
        XCENT : float
            see IOAPI GRIDDESC documentation
        YCENT : float
            see IOAPI GRIDDESC documentation
        GDTYP : float
            see IOAPI GRIDDESC documentation
        endian : string
            'big' or 'little' usually only if mistaken compile
        chemparam : None or string
            used to identify gases and aerosols

        Returns
        -------
        outf : uamiv
            PseudoNetCDFFile populated from file
        """
        if chemparam is None:
            self._aerosol_names = None
        else:
            self._aerosol_names = get_chemparam_names(chemparam)['aerosol']

        self.__endianprefix = dict(big='>', little='<')[endian]
        self._make_header_fmt()
        self.__rffile = rf
        self.__mode = mode

        self.createDimension('DATE-TIME', 2)

        self.__readheader()

        # Add IOAPI metavariables
        self.NLAYS = len(self.dimensions['LAY'])
        self.NROWS = len(self.dimensions['ROW'])
        self.NCOLS = len(self.dimensions['COL'])
        self.NVARS = len(self.dimensions['VAR'])
        self.NSTEPS = len(self.dimensions['TSTEP'])
        varlist = "".join([i.ljust(16) for i in self.__var_names__])
        setattr(self, 'VAR-LIST', varlist)
        self.NAME = self.__emiss_hdr['name'][0, :, 0].copy().view('S10')[
            0].decode()
        self.NOTE = self.__emiss_hdr['note'][0, :, 0].copy().view('S60')[
            0].decode()
        self.ITZON = self.__emiss_hdr['itzon'][0]
        self.FTYPE = 1
        self.VGTYP = 2
        self.VGTOP = 10000.
        self.VGLVLS = linspace(0, 1, self.NLAYS + 1)[::-1]
        self.GDNAM = "CAMx            "
        self.UPNAM = "CAMx            "
        self.FILEDESC = "CAMx            "
        # Create variables
        self.variables = PseudoNetCDFVariables(
            self.__variables, ['TFLAG', 'ETFLAG'] + self.__var_names__)
        tflag = ConvertCAMxTime(self.__memmap__['DATE']['BDATE'],
                                self.__memmap__['DATE']['BTIME'],
                                self.NVARS)
        etflag = ConvertCAMxTime(self.__memmap__['DATE']['EDATE'],
                                 self.__memmap__['DATE']['ETIME'],
                                 self.NVARS)
        tflagv = self.createVariable('TFLAG', 'i',
                                     ('TSTEP', 'VAR', 'DATE-TIME'),
                                     values=tflag, units='DATE-TIME',
                                     long_name='TFLAG'.ljust(16),
                                     var_desc='TFLAG'.ljust(80))
        etflagv = self.createVariable('ETFLAG', 'i',
                                      ('TSTEP', 'VAR', 'DATE-TIME'),
                                      values=etflag, units='DATE-TIME',
                                      long_name='ETFLAG'.ljust(16),
                                      var_desc='Ending TFLAG'.ljust(80))

        self.SDATE, self.STIME = self.variables['TFLAG'][0, 0, :]
        self.TSTEP = etflagv[0, 0, 1] - tflagv[0, 0, 1]
        if P_ALP is not None:
            self.P_ALP = P_ALP
        if P_BET is not None:
            self.P_BET = P_BET
        if P_GAM is not None:
            self.P_GAM = P_GAM
        if XCENT is not None:
            self.XCENT = XCENT
        if YCENT is not None:
            self.YCENT = YCENT
        if GDTYP is not None:
            self.GDTYP = GDTYP
        # try:
        #     add_cf_from_ioapi(self)
        # except Exception as e:
        #     warn(repr(e))
        #     pass

    def __checkfilelen(self):
        f = open(self.__rffile, 'rb')
        f.seek(0, 2)
        flen = f.tell()
        f.close()
        return flen

    @classmethod
    def isMine(cls, path):
        self = uamiv.__new__(uamiv)
        cls._make_header_fmt(self, '>')
        offset = 0
        emiss_hdr = memmap(
            path, mode='r', dtype=self.__emiss_hdr_fmt, shape=1, offset=offset)
        name = emiss_hdr['name'][0, :, 0].copy().view('S10')[
            0].decode().strip()
        if name in ('BOUNDARY', 'PTSOURCE'):
            return False

        if (
            not emiss_hdr['SPAD'] == emiss_hdr['EPAD'] and
            emiss_hdr['SPAD'] == emiss_hdr.dtype.itemsize - 8
        ):
            return False
        offset += emiss_hdr.dtype.itemsize * emiss_hdr.size

        grid_hdr = memmap(
            path, mode='r', dtype=self.__grid_hdr_fmt, shape=1, offset=offset)
        if (
            not grid_hdr['SPAD'] == grid_hdr['EPAD'] and
            grid_hdr['SPAD'] == grid_hdr.dtype.itemsize - 8
        ):
            return False
        offset += grid_hdr.dtype.itemsize * grid_hdr.size
        cell_hdr = memmap(
            path, mode='r', dtype=self.__cell_hdr_fmt, shape=1, offset=offset)
        if (
            not cell_hdr['SPAD'] == cell_hdr['EPAD'] and
            cell_hdr['SPAD'] == cell_hdr.dtype.itemsize - 8
        ):
            return False
        return name in ('AIRQUALITY', 'EMISSIONS', 'INSTANT', 'AVERAGE')

    def __readheader(self):
        ep = self.__endianprefix
        offset = 0
        self.__emiss_hdr = memmap(
            self.__rffile, mode=self.__mode, dtype=self.__emiss_hdr_fmt,
            shape=1, offset=offset)
        nspec = self.__emiss_hdr['nspec'][0]
        offset += self.__emiss_hdr.dtype.itemsize * self.__emiss_hdr.size

        self.__grid_hdr = memmap(
            self.__rffile, mode=self.__mode, dtype=self.__grid_hdr_fmt,
            shape=1, offset=offset)

        self.XORIG = self.__grid_hdr['xorg'][0]
        self.YORIG = self.__grid_hdr['yorg'][0]
        self.XCELL = self.__grid_hdr['delx'][0]
        self.YCELL = self.__grid_hdr['dely'][0]
        self.PLON = plon = self.__grid_hdr['plon'][0]
        self.PLAT = plat = self.__grid_hdr['plat'][0]
        self.TLAT1 = tlat1 = self.__grid_hdr['tlat1'][0]
        self.TLAT2 = tlat2 = self.__grid_hdr['tlat2'][0]
        self.IUTM = iutm = self.__grid_hdr['iutm'][0]
        self.ISTAG = self.__grid_hdr['istag'][0]
        self.CPROJ = cproj = self.__grid_hdr['iproj'][0]
        if hasattr(self, 'GDTYP'):
            GDTYPE = self.GDTYP
        else:
            GDTYPE = self.GDTYP = {0: 1, 1: 5, 2: 2, 3: 6}[cproj]
        if (
            cproj == 0 or
            not all([x == 0 for x in [plon, plat, tlat1, tlat2, iutm, cproj]])
        ):
            # Map CAMx projection constants to IOAPI
            self.XCENT = plon
            self.YCENT = plat
            if GDTYPE in (1, 2):
                self.P_ALP = tlat1
                self.P_BET = tlat2
                self.P_GAM = plon
            elif GDTYPE == 5:
                self.P_ALP = iutm
                self.P_BET = 0.
                self.P_GAM = 0.
            elif GDTYPE == 6:
                self.P_ALP = {90: 1, -90: -1}[plat]
                self.P_BET = tlat1
                self.P_GAM = plon
            else:
                raise ValueError('Unknown projection')

        nx = self.__grid_hdr['nx'][0]
        ny = self.__grid_hdr['ny'][0]
        nz = max(self.__grid_hdr['nz'], array([1]))[0]

        offset += self.__grid_hdr.dtype.itemsize * self.__grid_hdr.size
        self.__cell_hdr = memmap(
            self.__rffile, mode=self.__mode, dtype=self.__cell_hdr_fmt,
            shape=1, offset=offset)

        offset += self.__cell_hdr.dtype.itemsize * self.__cell_hdr.size + 4
        self.__spc_hdr = memmap(self.__rffile, mode=self.__mode,
                                dtype=self.__spc_fmt, shape=nspec,
                                offset=offset)

        offset += self.__spc_hdr.dtype.itemsize * self.__spc_hdr.size + 4

        date_time_fmt = dtype(dict(
            names=['SPAD', 'BDATE', 'BTIME', 'EDATE', 'ETIME', 'EPAD'],
            formats=['i', 'i', 'f', 'i', 'f', 'i'])).newbyteorder(ep)
        date_time_block_size = 6
        spc_1_lay_fmt = dtype(dict(
            names=['SPAD', 'IONE', 'SPC', 'DATA', 'EPAD'],
            formats=['i', 'i', '(10,4)S1',
                     '(%d,%d)f' % (ny, nx), 'i'])).newbyteorder(ep)
        spc_1_lay_block_size = 13 + nx * ny
        spc_3d_fmt = dtype((spc_1_lay_fmt, (nz,)))

        # Get species names from spc_hdr
        var_names = [spc[:, 0].copy().view('S10')[0] for spc in self.__spc_hdr]
        var_names = [v.decode() if hasattr(v, 'decode')
                     else v for v in var_names]
        self.__var_names__ = [''.join(v).strip() for v in var_names]

        data_block_fmt = dtype(dict(
            names=['DATE'] + self.__var_names__,
            formats=[date_time_fmt] + [spc_3d_fmt] * nspec))

        data_block_size = (date_time_block_size + nspec * nz *
                           spc_1_lay_block_size)
        f = open(self.__rffile)
        f.seek(0, 2)
        size = f.tell()
        f.close()
        del f
        ntimes = float(size - offset) / 4. / data_block_size
        if int(ntimes) != ntimes:
            raise ValueError(
                ("Partial time output (%f times); partial time indicates " +
                 "incomplete file.") % ntimes)
        ntimes = int(ntimes)

        self.createDimension('LAY', nz)
        self.createDimension('COL', nx)
        self.createDimension('ROW', ny)
        tstep = self.createDimension('TSTEP', ntimes)
        tstep.setunlimited(True)
        self.createDimension('VAR', nspec)

        self.__memmap__ = memmap(self.__rffile, mode=self.__mode,
                                 dtype=data_block_fmt, offset=offset)

    def __variables(self, k):
        dimensions = ('TSTEP', 'LAY', 'ROW', 'COL')
        outvals = self.__memmap__[k]['DATA']
        units = get_uamiv_units(self.NAME, k, self._aerosol_names)

        return PseudoIOAPIVariable(self, k, 'f', dimensions, values=outvals,
                                   units=units)


class TestMemmap(unittest.TestCase):
    def runTest(self):
        pass

    def setUp(self):
        pass

    def testAvg(self):
        import PseudoNetCDF.testcase
        self.assertTrue(uamiv.isMine(
            PseudoNetCDF.testcase.camxfiles_paths['uamiv']))
        emissfile = uamiv(PseudoNetCDF.testcase.camxfiles_paths['uamiv'])
        emissfile.variables['TFLAG']
        v = emissfile.variables['NO2']
        checkv = array(
            [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
             0.00000000e+00, 0.00000000e+00, 1.24175494e-04, 2.79196858e-04,
             1.01672206e-03, 4.36782313e-04, 0.00000000e+00, 1.54810550e-04,
             3.90250643e-04, 6.18023798e-04, 3.36963218e-04, 0.00000000e+00,
             1.85579920e-04, 1.96825975e-04, 2.16468165e-04, 2.19882189e-04],
            dtype='f').reshape(1, 1, 4, 5)
        self.assertTrue((v == checkv).all())

    def testClose(self):
        import PseudoNetCDF.testcase
        emissfile = uamiv(PseudoNetCDF.testcase.camxfiles_paths['uamiv'])
        emissfile.close()

    def testSync(self):
        import PseudoNetCDF.testcase
        emissfile = uamiv(PseudoNetCDF.testcase.camxfiles_paths['uamiv'])
        emissfile.sync()


if __name__ == '__main__':
    unittest.main()



__all__ = ['pncscatter']

import matplotlib.pyplot as plt
import numpy as np


def pncscatter(args):
    ifiles = args.ifiles
    fig = plt.figure()
    if len(args.figure_keywords) > 0:
        plt.setp(fig, **args.figure_keywords)
    sax = fig.add_subplot(111)  # add_axes([.1, .15, .8, .8])
    if len(args.axes_keywords) > 0:
        plt.setp(sax, **args.axes_keywords)

    ifilex = ifiles[0]
    ifiley = ifiles[1]

    for target in args.variables:
        sax.set_xlabel('Time (UTC)')
        varx = ifilex.variables[target]
        varxdesc = getattr(varx, 'description', None)
        vary = ifiley.variables[target]
        varydesc = getattr(vary, 'description', None)
        sax.set_ylabel(varydesc)
        sax.set_xlabel(varxdesc)
        del sax.lines[:]
        vmin = np.minimum(varx[:].min(), vary[:].min())
        vmax = np.maximum(varx[:].max(), vary[:].max())
        sax.plot([vmin, vmax], [vmin, vmax], color='k')
        sax.plot(varx[:].ravel(), vary[:].ravel(),
                 marker='o', ls='none', markeredgecolor='none')
        sax.set_xlim(vmin, vmax)
        sax.set_ylim(vmin, vmax)
        # plt.setp(sax.xaxis.get_ticklabels(),rotation = 45)
        figpath = args.outpath + target + '.' + args.figformat
        for pc in args.plotcommands:
            exec(pc)
        fig.savefig(figpath)
        if args.verbose > 0:
            print('Saved fig', figpath)



from __future__ import print_function

__all__ = ['mapplot', 'profile', 'tileplot',
           'presslon', 'presslat', 'timeseries', 'OptionDict']

from warnings import warn
import os

import numpy as np

import PseudoNetCDF.coordutil as cu


_pre_code = 'plt.close(); plt.rcParams["image.cmap"] = "jet"'
_before_code = 'plt.figure(); plt.interactive(True); '
_after_code = 'ax.set_title(varkey); plt.draw()'
_post_code = ''
_coastlines_opt = True
_countries_opt = True
_states_opt = True
_counties_opt = False
_coordkeys = ('time', 'latitude', 'longitude', 'latitude_bounds',
              'longitude_bounds', 'time_bounds', 'tau0', 'tau1',
              'TFLAG')


class OptionDict(dict):
    def __init__(self, *args, **kwds):
        """
        coastlines = %(_coastlines_opt)s
        countries = %(_countries_opt)s
        states = %(_states_opt)s
        counties = %(_counties_opt)s
        pre_txt = Code to run before any plots (%(_pre_code)s)
        before_txt = Code to run before each variable-plot (%(_before_code)s)
        after_txt = Code to run after each plot (%(_after_code)s)
        post_txt = Code to run after all plots (%(_post_code)s)
        """ % globals()
        dict.__init__(self, coastlines=_coastlines_opt,
                      countries=_countries_opt, states=_states_opt,
                      counties=_counties_opt, pre_txt=_pre_code,
                      before_txt=_before_code, after_txt=_after_code,
                      post_txt=_post_code)
        dict.__init__(self, *args, **kwds)
        if 'outpath' not in self:
            raise KeyError('outpath is a required option')

    def __getattr__(self, key):
        return self.get(key, False)


defaultoption = OptionDict(outpath='pncview')


class TkApp:
    def __init__(self, ncffile, options):
        try:
            from Tkinter import Checkbutton, Frame, Label, Scrollbar
            from Tkinter import Listbox, Button, IntVar, Tk, VERTICAL
            from Tkinter import EXTENDED, END, N, S, SINGLE, Entry
            from Tkinter import StringVar, Text, DISABLED, LEFT
        except Exception:
            try:
                from tkinter import Checkbutton, Frame, Label, Scrollbar
                from tkinter import Listbox, Button, IntVar, Tk, VERTICAL
                from tkinter import EXTENDED, END, N, S, SINGLE, Entry
                from tkinter import StringVar, Text, DISABLED, LEFT
            except Exception:
                warn('tkinter unavailable')

        master = self.root = Tk()
        self.ncffile = ncffile
        self.options = options
        self.plotted_variables = set()
        frame = Frame(master)
        frame.grid(row=0)
        codeframe = Frame(master)
        codeframe.grid(row=1)
        metaframe = Frame(master)
        metaframe.grid(row=2)
        goframe = Frame(frame)
        goframe.grid(column=3, row=1)

        var_label = Label(frame, text='Select Variable')
        var_label.grid(column=0, row=0)
        var_scrollbar = Scrollbar(frame, orient=VERTICAL)
        var_scrollbar.grid(column=1, row=1, sticky=N + S)
        self.var = Listbox(frame, selectmode=EXTENDED,
                           exportselection=0,
                           yscrollcommand=var_scrollbar.set)
        self.var.grid(column=0, row=1)
        var_scrollbar.config(command=self.var.yview)

        what_to_do = Label(frame, text='Execute')
        what_to_do.grid(column=2, row=0)
        self.method_list = Listbox(frame, selectmode=SINGLE, exportselection=0)
        self.method_list.grid(column=2, row=1)
        self.pre_txt = StringVar()
        pre_label = Label(codeframe, text='Before any figures, execute code')
        self.pre_txt.set(_pre_code)
        pre_label.grid(row=2, sticky='W')
        self.pre = Entry(codeframe, width=120, textvariable=self.pre_txt)
        self.pre.grid(row=3, sticky='E')

        self.before_txt = StringVar()
        self.before_txt.set(_before_code)
        before_label = Label(
            codeframe, text='Before each figure, execute code')
        before_label.grid(row=4, sticky='W')
        self.before = Entry(codeframe, width=120, textvariable=self.before_txt)
        self.before.grid(row=5, sticky='E')

        self.after_txt = StringVar()
        self.after_txt.set(_after_code)
        after_label = Label(codeframe, text='After each figure, execute code')
        after_label.grid(row=6, sticky='W')
        self.after = Entry(codeframe, width=120, textvariable=self.after_txt)
        self.after.grid(row=7, sticky='E')

        self.post_txt = StringVar()
        self.post_txt.set(_post_code)
        post_label = Label(codeframe, text='After all figures, execute code')
        post_label.grid(row=8, sticky='W')
        self.post = Entry(codeframe, width=120, textvariable=self.post_txt)
        self.post.grid(row=9, sticky='E')

        options_label = Label(goframe, text='Options:')
        options_label.grid(column=0, row=1, sticky='W')
        self.logscale = IntVar()
        self.logscale.set(0)
        c = Checkbutton(goframe, text="log-scale?", variable=self.logscale)
        c.grid(column=0, row=2, sticky='W')

        self.coastlines = IntVar()
        self.coastlines.set(_coastlines_opt)
        coastlines = Checkbutton(
            goframe, text="coastlines?", variable=self.coastlines,
            justify=LEFT)
        coastlines.grid(column=0, row=3, sticky='W')

        self.countries = IntVar()
        self.countries.set(_countries_opt)
        countries = Checkbutton(
            goframe, text="countries?", variable=self.countries, justify=LEFT)
        countries.grid(column=0, row=4, sticky='W')

        self.states = IntVar()
        self.states.set(_states_opt)
        states = Checkbutton(goframe, text="states?",
                             variable=self.states, justify=LEFT)
        states.grid(column=0, row=5, sticky='W')

        self.counties = IntVar()
        self.counties.set(_counties_opt)
        counties = Checkbutton(goframe, text="counties?",
                               variable=self.counties, justify=LEFT)
        counties.grid(column=0, row=6, sticky='W')

        self.execute_button = Button(
            goframe, text="Make Figure", command=self.execute)
        self.execute_button.grid(row=0, column=0, sticky='W')

        self.methods = ['mapplot', 'presslat', 'presslon', 'time-lat',
                        'profile', 'timeseries', 'pressx', 'tileplot', 'plot']
        method_labels = ['lat-lon', 'press-lat', 'press-lon', 'time-lat',
                         'Vertical Profile', 'Time Series', 'press-? (2-D)',
                         'Tile Plot (2-D)', 'Plot (1-D)']
        for method in method_labels:
            self.method_list.insert(END, method)
        var_keys = [k for k, v in self.ncffile.variables.items()
                    if k not in _coordkeys]
        var_keys.sort()
        self.vars = []
        for spc in var_keys:
            self.var.insert(END, spc)
            self.vars.append(spc)

        meta_label = Label(metaframe, text='Common Data Language Header:')
        meta_label.grid(column=0, row=0, sticky='W')
        meta_scrollbar = Scrollbar(metaframe, orient=VERTICAL)
        meta_scrollbar.grid(column=1, row=1, sticky=N + S)
        self.meta = Text(metaframe, height=10, width=118, bg='white',
                         relief='flat', yscrollcommand=meta_scrollbar.set)
        self.meta.grid(column=0, row=1, sticky='W')
        from PseudoNetCDF.pncdump import pncdump
        try:
            from StringIO import StringIO
        except ImportError:
            from io import StringIO
        pdump = StringIO("")
        pncdump(self.ncffile, header=True, outfile=pdump, )
        pdump.seek(0, 0)
        self.meta.insert(END, pdump.read())
        self.meta.config(state=DISABLED)
        help = Button(goframe, text='Help', command=self.help)
        help.grid(column=0, row=7)
        quit = Button(goframe, text='Quit', command=self.quit)
        quit.grid(column=0, row=8)
        master.mainloop()

    def help(self):
        print("pl is pylab: details at matplotlib;")

    def quit(self):
        self.root.destroy()

    def _get_var(self, list):
        items = list.curselection()
        try:
            items = map(int, items)
        except Exception:
            pass
        items = [self.vars[i] for i in items]
        return items

    def get_var(self):
        return self._get_var(self.var)

    def get_methods(self):
        items = self.method_list.curselection()
        try:
            items = map(int, items)
        except Exception:
            pass
        items = [self.methods[i] for i in items]
        return items

    def execute(self):
        os.system('clear')
        vars = self.get_var()
        self.plotted_variables = self.plotted_variables.union(vars)
        methods, = self.get_methods()
        self.options.logscale = bool(self.logscale.get())
        self.options.coastlines = bool(self.coastlines.get())
        self.options.countries = bool(self.countries.get())
        self.options.states = bool(self.states.get())
        self.options.counties = bool(self.counties.get())
        self.options.pre_txt = self.pre_txt.get()
        self.options.before_txt = self.before_txt.get()
        self.options.after_txt = self.after_txt.get()
        self.options.post_txt = self.post_txt.get()
        plotwithopts(self.ncffile, methods, vars, self.options)


def plotwithopts(ifile, method, vars, options=defaultoption):
    from PseudoNetCDF.sci_var import getvarpnc
    from PseudoNetCDF.pncgen import pncgen
    import matplotlib.pyplot as plt

    # dummy assignment so that flake8 sees plt as used
    # plt is loaded into the environment for exec
    varkey = plt

    exec(options.pre_txt)
    for varkey in vars:
        figpath = eval(method)(ifile=ifile, varkey=varkey, options=options,
                               before=options.before_txt,
                               after=options.after_txt)
        pncgen(getvarpnc(ifile, list(vars) + list(_coordkeys)),
               figpath + '.nc', verbose=0)
    exec(options.post_txt)


def StartTk(ncffile, options):
    TkApp(ncffile, options)


def pncview(ifile, options):
    # add a gui for plotting
    return StartTk(ifile, options)


def gettime(ifile):
    from PseudoNetCDF import PseudoNetCDFVariable
    from datetime import datetime, timedelta
    if 'time' in ifile.variables:
        time = ifile.variables['time']
        unit = time.units
        tunit, datestr = unit.split(' since ')
        sdate = datetime.strptime(datestr.replace(
            ' UTC', ''), '%Y-%m-%d %H:%M:%S')
        time = np.array([sdate + timedelta(**{tunit: t}) for t in time[:]])
        unit = 'time'
    elif 'TFLAG' in ifile.variables:
        x = ifile.variables['TFLAG'][:].copy()
        for axi in range(1, x.ndim - 1):
            x = x[:, 0]
        time = np.array(
            [(datetime.strptime('%07d %06d' % (d, t), '%Y%j %H%M%S'))
             for d, t in x])
        unit = 'time'
    elif 'time' in ifile.dimensions:
        time = np.arange(len(ifile.dimensions['time']))
        unit = 'steps'
    elif 'TSTEP' in ifile.dimensions:
        time = np.arange(1, len(ifile.dimensions['TSTEP']) + 1)
        unit = 'steps'
    else:
        raise KeyError('No time found')
    return PseudoNetCDFVariable(None, 'time', 'f', ('time',),
                                values=time[:], units=unit)


def timeseries(ifile, varkey, options, before='', after=''):
    import pylab as pl
    outpath = getattr(options, 'outpath', '.')
    time = gettime(ifile)
    var = ifile.variables[varkey]
    dims = [(k, l) for l, k in zip(var[:].shape, var.dimensions) if l > 1]
    if len(dims) > 1:
        raise ValueError(
            'Time series can have 1 non-unity dimensions; got %d - %s' %
            (len(dims), str(dims)))
    exec(before)
    ax = pl.gca()
    print(varkey, end='')
    if options.logscale:
        ax.set_yscale('log')

    ax.plot_date(time[:].squeeze(), var[:].squeeze())
    ax.set_xlabel(time.units.strip())
    ax.set_ylabel(getattr(var, 'standard_name',
                          varkey).strip() + ' ' + var.units.strip())
    fmt = 'png'
    figpath = os.path.join(outpath + '_ts_' + varkey + '.' + fmt)
    exec(after)
    pl.savefig(figpath)
    print('Saved fig', figpath)
    return figpath


def plot(ifile, varkey, options, before='', after=''):
    import pylab as pl
    outpath = getattr(options, 'outpath', '.')
    var = ifile.variables[varkey]
    dims = [(k, l) for l, k in zip(var[:].shape, var.dimensions) if l > 1]
    if len(dims) > 1:
        raise ValueError(
            'Plots can have only 1 non-unity dimensions; got %d - %s' %
            (len(dims), str(dims)))
    exec(before)
    ax = pl.gca()
    print(varkey, end='')
    if options.logscale:
        ax.set_yscale('log')

    ax.plot(var[:].squeeze())
    ax.set_xlabel('unknown')
    ax.set_ylabel(getattr(var, 'standard_name',
                          varkey).strip() + ' ' + var.units.strip())
    fmt = 'png'
    figpath = os.path.join(outpath + '_1d_' + varkey + '.' + fmt)
    exec(after)
    pl.savefig(figpath)
    print('Saved fig', figpath)
    return figpath


def pressx(ifile, varkey, options, before='', after=''):
    import matplotlib.pyplot as plt
    from matplotlib.colors import Normalize, LogNorm
    outpath = getattr(options, 'outpath', '.')
    vert = cu.getpresbnds(ifile)
    var = ifile.variables[varkey]
    dims = [(k, l) for l, k in zip(var[:].shape, var.dimensions) if l > 1]
    if len(dims) > 2:
        raise ValueError(
            'Press-x can have 2 non-unity dimensions; got %d - %s' %
            (len(dims), str(dims)))
    if options.logscale:
        norm = LogNorm()
    else:
        norm = Normalize()
    exec(before)
    ax = plt.gca()
    print(varkey, end='')
    vals = var[:].squeeze()
    x = np.arange(vals.shape[1])
    patches = ax.pcolor(x, vert, vals, norm=norm)
    # ax.set_xlabel(X.units.strip())
    # ax.set_ylabel(Y.units.strip())
    plt.colorbar(patches)
    ax.set_ylim(vert.max(), vert.min())
    ax.set_xlim(x.min(), x.max())
    fmt = 'png'
    figpath = os.path.join(outpath + '_PRESX_' + varkey + '.' + fmt)
    exec(after)
    plt.savefig(figpath)
    print('Saved fig', figpath)
    return figpath


def presslat(ifile, varkey, options, before='', after=''):
    import matplotlib.pyplot as plt
    from matplotlib.colors import Normalize, LogNorm
    outpath = getattr(options, 'outpath', '.')
    vert = cu.getpresbnds(ifile)
    lat, latunit = cu.getlatbnds(ifile)
    lat = np.append(lat.squeeze()[..., :2].mean(
        1), lat.squeeze()[-1, 2:].mean(0))
    var = ifile.variables[varkey]
    dims = [(k, l) for l, k in zip(var[:].shape, var.dimensions) if l > 1]
    if len(dims) > 2:
        raise ValueError(
            'Press-lat can have 2 non-unity dimensions; got %d - %s' %
            (len(dims), str(dims)))
    if options.logscale:
        norm = LogNorm()
    else:
        norm = Normalize()
    exec(before)
    ax = plt.gca()
    print(varkey, end='')
    patches = ax.pcolor(lat, vert, var[:].squeeze(), norm=norm)
    # ax.set_xlabel(X.units.strip())
    # ax.set_ylabel(Y.units.strip())
    cbar = plt.colorbar(patches)
    vunit = getattr(var, 'units', 'unknown').strip()
    cbar.set_label(varkey + ' (' + vunit + ')')
    cbar.ax.text(.5, 1, '%.2g' % var[:].max(
    ), horizontalalignment='center', verticalalignment='bottom')
    cbar.ax.text(.5, 0, '%.2g' % var[:].min(
    ), horizontalalignment='center', verticalalignment='top')
    ax.set_ylim(vert.max(), vert.min())
    ax.set_xlim(lat.min(), lat.max())
    fmt = 'png'
    figpath = os.path.join(outpath + '_PRESSLAT_' + varkey + '.' + fmt)
    exec(after)
    plt.savefig(figpath)
    print('Saved fig', figpath)
    return figpath


def presslon(ifile, varkey, options, before='', after=''):
    import matplotlib.pyplot as plt
    from matplotlib.colors import Normalize, LogNorm
    outpath = getattr(options, 'outpath', '.')
    vert = cu.getpresbnds(ifile)
    lon, lonunit = cu.getlonbnds(ifile)
    lon = np.append(lon.squeeze()[..., [0, 3]].mean(
        1), lon.squeeze()[-1, [1, 2]].mean(0))
    var = ifile.variables[varkey]
    dims = [(k, l) for l, k in zip(var[:].shape, var.dimensions) if l > 1]
    if len(dims) > 2:
        raise ValueError(
            'Press-lon plots can have 2 non-unity dimensions; got %d - %s' %
            (len(dims), str(dims)))
    if options.logscale:
        norm = LogNorm()
    else:
        norm = Normalize()
    exec(before)
    ax = plt.gca()
    print(varkey, end='')
    patches = ax.pcolor(lon, vert, var[:].squeeze(), norm=norm)
    # ax.set_xlabel(X.units.strip())
    # ax.set_ylabel(Y.units.strip())
    cbar = plt.colorbar(patches)
    vunit = getattr(var, 'units', 'unknown').strip()
    cbar.set_label(varkey + ' (' + vunit + ')')
    cbar.ax.text(.5, 1, '%.2g' % var[:].max(
    ), horizontalalignment='center', verticalalignment='bottom')
    cbar.ax.text(.5, 0, '%.2g' % var[:].min(
    ), horizontalalignment='center', verticalalignment='top')

    ax.set_ylim(vert.max(), vert.min())
    ax.set_xlim(lon.min(), lon.max())
    fmt = 'png'
    figpath = os.path.join(outpath + '_PRESLON_' + varkey + '.' + fmt)
    exec(after)
    plt.savefig(figpath)
    print('Saved fig', figpath)
    return figpath


def tileplot(ifile, varkey, options, before='', after=''):
    import matplotlib.pyplot as plt
    from matplotlib.colors import Normalize, LogNorm
    outpath = getattr(options, 'outpath', '.')
    var = ifile.variables[varkey]
    if options.logscale:
        norm = LogNorm()
    else:
        norm = Normalize()
    exec(before)
    ax = plt.gca()
    print(varkey, end='')
    dims = [(k, l) for l, k in zip(var[:].shape, var.dimensions) if l > 1]
    if len(dims) > 2:
        raise ValueError(
            'Tile plots can have 2 non-unity dimensions; got %d - %s' %
            (len(dims), str(dims)))
    patches = ax.pcolor(var[:].squeeze(), norm=norm)
    ax.set_xlim(0, var[:].squeeze().shape[1])
    ax.set_ylim(0, var[:].squeeze().shape[0])
    ax.set_xlabel(dims[1][0])
    ax.set_ylabel(dims[0][0])
    # ax.set_xlabel(X.units.strip())
    # ax.set_ylabel(Y.units.strip())
    cbar = plt.colorbar(patches)
    vunit = getattr(var, 'units', 'unknown').strip()
    cbar.set_label(varkey + ' (' + vunit + ')')
    cbar.ax.text(.5, 1, '%.2g' % var[:].max(
    ), horizontalalignment='center', verticalalignment='bottom')
    cbar.ax.text(.5, 0, '%.2g' % var[:].min(
    ), horizontalalignment='center', verticalalignment='top')
    fmt = 'png'
    figpath = os.path.join(outpath + '_2D_' + varkey + '.' + fmt)
    exec(after)
    plt.savefig(figpath)
    print('Saved fig', figpath)
    return figpath


def minmaxmean(ax, vals, vertcrd, **kwds):
    minval = vals.min(1)
    meanval = vals.mean(1)
    maxval = vals.max(1)
    kwds.setdefault('edgecolor', 'k')
    kwds.setdefault('facecolor', 'grey')
    kwds.setdefault('alpha', 1)
    kwds.setdefault('ls', 'solid')
    linekwds = kwds.copy()
    linekwds['color'] = linekwds.pop('edgecolor')
    linekwds.pop('facecolor')
    linekwds.pop('alpha')
    linekwds.pop('ls')
    fillkwds = kwds.copy()

    line, = ax.plot(meanval, vertcrd, **linekwds)

    x = np.ma.concatenate([minval[:vertcrd.size], maxval[:vertcrd.size][::-1]])
    y = np.ma.concatenate([vertcrd[:], vertcrd[::-1]])
    mask = x.mask | y.mask
    x = np.ma.masked_where(mask, x).compressed()
    y = np.ma.masked_where(mask, y).compressed()
    range, = ax.fill(x, y, **fillkwds)
    return line, range


def profile(ifile, varkey, options, before='', after=''):
    import matplotlib.pyplot as plt
    print(varkey, end='')
    outpath = getattr(options, 'outpath', '.')
    try:
        vert = cu.getpresmid(ifile)
        vunit = 'Pa'
    except Exception:
        vert = cu.getsigmamid(ifile)
        vunit = r'\sigma'
    var = ifile.variables[varkey]

    dims = list(var.dimensions)
    for knownvert in ['layer', 'LAY'] + ['layer%d' % i for i in range(72)]:
        if knownvert in dims:
            vidx = dims.index(knownvert)
            break
    else:
        raise KeyError("No known vertical coordinate; got %s" % str(dims))
    vert = vert[:var[:].shape[vidx]]
    units = var.units.strip()
    vals = np.rollaxis(var[:], vidx, start=0).view(
        np.ma.MaskedArray).reshape(vert.size, -1)
    ax = plt.gca()
    minmaxmean(ax, vals, vert)
    ax.set_xlabel(varkey + ' (' + units + ')')
    ax.set_ylabel(vunit)
    ax.set_ylim(vert.max(), vert.min())
    if options.logscale:
        ax.set_xscale('log')
    fmt = 'png'
    figpath = os.path.join(outpath + '_profile_' + varkey + '.' + fmt)
    exec(after)
    plt.savefig(figpath)
    print('Saved fig', figpath)
    return figpath


def mapplot(ifile, varkey, options, before='', after=''):
    """
    ifile - a pseudonetcdf file
    varkey - the variable to plot
    options - argparse name space with mapping options
    """
    import matplotlib.pyplot as plt
    from matplotlib.colors import Normalize, LogNorm
    outpath = getattr(options, 'outpath', '.')
    map = cu.getmap(ifile)
    if map.projection == 'cyl':
        latb, latunit = cu.getlatbnds(ifile)[:]
        lonb, lonunit = cu.getlonbnds(ifile)[:]
    else:
        latb, latunit = cu.getybnds(ifile)[:]
        lonb, lonunit = cu.getxbnds(ifile)[:]
    if latb.ndim == lonb.ndim and lonb.ndim == 2:
        LON, LAT = lonb, latb
    else:
        LON, LAT = np.meshgrid(lonb, latb)

    ax = plt.gca()
    if options.logscale:
        norm = LogNorm()
    else:
        norm = Normalize()
    var = ifile.variables[varkey]
    exec(before)
    ax = plt.gca()
    vunit = getattr(var, 'units', 'unknown').strip()
    print(varkey, end='')
    try:
        if options.coastlines:
            map.drawcoastlines(ax=ax)
        if options.countries:
            map.drawcountries(ax=ax)
        if options.states:
            map.drawstates(ax=ax)
        if options.counties:
            map.drawcounties(ax=ax)
    except Exception:
        print('nomap')
        pass
    patches = map.pcolor(LON, LAT, var[:].squeeze(), norm=norm, ax=ax)
    if lonunit == 'x (LCC m)':
        ax.xaxis.get_major_formatter().set_scientific(True)
        ax.xaxis.get_major_formatter().set_powerlimits((-3, 3))
    if latunit == 'y (LCC m)':
        ax.yaxis.get_major_formatter().set_scientific(True)
        ax.yaxis.get_major_formatter().set_powerlimits((-3, 3))
    ax.set_xlabel(lonunit)
    ax.set_ylabel(latunit)
    height = LAT.max() - LAT.min()
    width = LON.max() - LON.min()
    if width > height:
        orientation = 'horizontal'
    else:
        orientation = 'vertical'
    cbar = plt.gcf().colorbar(patches, orientation=orientation)
    cbar.set_label(varkey + ' (' + vunit + ')')
    if orientation == 'vertical':
        cbar.ax.text(.5, 1, '%.2g' % var[:].max(
        ), horizontalalignment='center', verticalalignment='bottom')
        cbar.ax.text(.5, 0, '%.2g' % var[:].min(
        ), horizontalalignment='center', verticalalignment='top')
    else:
        cbar.ax.text(1, .5, '%.2g' % var[:].max(
        ), verticalalignment='center', horizontalalignment='left')
        cbar.ax.text(0, .5, '%.2g' % var[:].min(
        ), verticalalignment='center', horizontalalignment='right')
    try:
        cbar.formatter.set_scientific(True)
        cbar.formatter.set_powerlimits((-3, 3))
    except Exception:
        pass
    cbar.update_ticks()
    fmt = 'png'
    figpath = os.path.join(outpath + '_map_' + varkey + '.' + fmt)
    exec(after)
    plt.savefig(figpath)
    print('Saved fig', figpath)
    return figpath


def main():
    import matplotlib.pyplot as plt
    from .pncparse import pncparse
    ifiles, options = pncparse(
        has_ofile=True, plot_options=True, interactive=False)
    if len(ifiles) != 1:
        raise IOError(
            'pncview can operate on only 1 file; user requested %d' %
            len(ifiles))
    ifile, = ifiles
    plt.interactive(True)
    for method_vars in options.plotcommands:
        pieces = method_vars.split(',')
        plotargs = [p for p in pieces if '=' not in p]
        plotkwds = [p for p in pieces if '=' in p]
        method, = plotargs[:1]
        vars = plotargs[1:]
        plotoptions = eval('OptionDict(outpath="%s",%s)' %
                           (options.outpath, ','.join(plotkwds)))
        print(plotoptions.logscale)
        plotwithopts(ifile, method, vars, plotoptions)
    plt.interactive(False)
    if len(options.plotcommands) == 0:
        pncview(ifile, options)


if __name__ == '__main__':
    main()



from __future__ import unicode_literals
import numpy as np
from PseudoNetCDF._getwriter import registerwriter


def ncf2landuse(ncffile, outpath):
    nland = len(ncffile.dimensions['LANDUSE'])
    nrows = len(ncffile.dimensions['ROW'])
    ncols = len(ncffile.dimensions['COL'])
    newstyle = getattr(ncffile, '_newstyle', True)
    dt3dfmt = '(%d, %d, %d)>f' % (nland, nrows, ncols)
    dt2dfmt = '(%d, %d)>f' % (nrows, ncols)
    if newstyle:
        _fland_dtype = np.dtype(dict(names=['SPAD1', 'KEY', 'EPAD1', 'SPAD2',
                                            'DATA', 'EPAD2'],
                                     formats=['>i', '8>S', '>i', '>i', dt3dfmt,
                                              '>i']))
        _other_dtype = np.dtype(dict(names=['SPAD1', 'KEY', 'EPAD1', 'SPAD2',
                                            'DATA', 'EPAD2'],
                                     formats=['>i', '8>S', '>i', '>i', dt2dfmt,
                                              '>i']))
    else:
        _fland_dtype = np.dtype(dict(names=['SPAD2', 'DATA', 'EPAD2'],
                                     formats=['>i', dt3dfmt, '>i']))
        _other_dtype = np.dtype(dict(names=['SPAD2', 'DATA', 'EPAD2'],
                                     formats=['>i', dt2dfmt, '>i']))

    outfile = open(outpath, 'wb')
    keys = [key
            for key in ['FLAND', 'VAR1', 'LAI', 'TOPO', 'LUCAT11', 'LUCAT26']
            if key in ncffile.variables.keys()]

    ludts = {'FLAND': _fland_dtype,
             'LUCAT11': _fland_dtype,
             'LUCAT26': _fland_dtype}

    def getempty(key):
        return np.empty(shape=(1,), dtype=ludts.get(key, _other_dtype))

    keyandvar = [(key, getempty(key)) for key in keys]
    for key, var in keyandvar:
        invar = ncffile.variables[key]
        if newstyle:
            if key == 'FLAND'.strip():
                key = 'LUCAT%02d' % nland

            var['SPAD1'] = 8
            var['KEY'] = key.ljust(8)
            var['EPAD1'] = 8

        var['SPAD2'][...] = invar.size * 4
        var['DATA'][...] = invar[:]
        var['EPAD2'][...] = invar.size * 4
        var.tofile(outfile)

    outfile.flush()
    return outfile


registerwriter('camxfiles.landuse', ncf2landuse)
registerwriter('landuse', ncf2landuse)



from PseudoNetCDF import PseudoNetCDFFile
import numpy as np
import matplotlib.pyplot as plt
import unittest


class ceilometerl2(PseudoNetCDFFile):
    """
    ceilometerhis is designed to read the "*.his"' files from
    the Vaisala ceilometers (CL31 and CL51), which is a level
    2 raw output
    """

    def __init__(self, path, dz=10.):
        """
        Parameters
        ----------
        path : path to output file
        dz : int or float
            level depth in meters
        """
        try:
            import pandas as pd
        except Exception:
            raise ImportError('ceilometerl2 requires pandas; ' +
                              'install pandas (e.g., pip install pandas)')
        self._path = path
        self._data = pd.read_csv(
            path, skiprows=1,
            parse_dates=['CREATEDATE']).rename(columns=lambda x: x.strip())
        self._data['BS_PROFILE'] = self._data['BS_PROFILE'].str.strip()
        nchar = len(self._data['BS_PROFILE'][0])
        nlays = int(nchar // 5)
        top = nlays * dz
        data = np.array(
            ''.join(self._data['BS_PROFILE']), dtype='c').reshape(-1, nlays, 5)
        BS_PROFILE = (np.vectorize(int)(data, 16) *
                      16**np.arange(5)[::-1]).sum(2)

        self.createDimension('time', data.shape[0]).setunlimited(True)
        self.createDimension('altitude_tops', nlays)
        self.createDimension('altitude_edges', nlays + 1)
        var = self.createVariable('time', 'd', ('time',))
        var[:] = self._data['UNIXTIME']
        var.description = 'time'
        var.units = 'seconds since 1970-01-01 00:00:00+0000'
        var = self.createVariable('PERIOD', 'i', ('time',))
        var[:] = self._data['PERIOD']
        var.units = 'seconds'
        altitude_tops = self.createVariable(
            'altitude_tops', 'd', ('altitude_tops',))
        altitude_tops.description = ("Profile bin altitude, meters above " +
                                     "ground level")
        altitude_tops.units = "meters"
        altitude_tops.valid_range = np.array([0., top])
        altitude_tops[:] = np.arange(dz, top + dz, dz)
        altitude_edges = self.createVariable(
            'altitude_edges', 'd', ('altitude_edges',))
        altitude_edges.units = "meters"
        altitude_edges.valid_range = np.array([0, top])
        altitude_edges.description = ("Profile bin altitude, meters above " +
                                      "ground level")
        altitude_edges[:] = np.arange(0, top + dz, dz)
        bscatprof = self.createVariable('blview_backscatter_profile', 'd',
                                        ('time', 'altitude_tops'),
                                        fill_value=-999)
        bscatprof.description = ("Backscatter profile at 910 nm not more " +
                                 "than 4,500 m above ground level")
        bscatprof.units = "10^-9/m/sr"
        bscatprof.valid_range = np.array([10, 540000])
        bscatprof[:] = np.ma.masked_greater(np.ma.masked_less(BS_PROFILE, 10),
                                            540000)

    def plot(self, fig_kw={'figsize': (12, 6)}, ax_kw=None, cax_kw=None,
             plot_kw=None):
        """
        Parameters
        ----------
        fig_kw : keywords for figure creation, default {'figsize': (12, 6)}
        ax_kw  : keywords for axes creation of the primary axis, default {}
        cax_kw  : keywords for axes creation of the colorbar axis, default {}
        plot_kw : keywords for axes creation of pseudo-color, default {}.
                    the default value for norm is added as
                    matplotlib.colors.LogNorm() if not provided

        Retruns
        -------
        fig : figure object produced

        Notes
        -----
        - fig_kw : see matplotlilb.pyplot.figure for options
        - ax_kw : and cax_kw see matptlotlib.pyplot.axes for options
                  except that ax_kw and cax_kw both can take an optional rect
                  keyword the rect will be used as the positional argument to
                  figure.add_axes
          - the default primary axis rect is [.075, .15, .8, .8]
          - the default colorbar axis rect starts 1% figure width to the
            right of the primary axis and takes 1/3 of the remaining space
        - plot_kw : see matplotlib.pyplot.pcolormesh for options
        """

        if ax_kw is None:
            ax_kw = {}

        if cax_kw is None:
            cax_kw = {}

        if plot_kw is None:
            plot_kw = {}

        timedelta = plt.matplotlib.dates.datetime.timedelta
        fig = plt.figure(**fig_kw)
        axrect = ax_kw.pop('rect', [.075, .15, .8, .8])
        ax = fig.add_axes(axrect, **ax_kw)
        # The default color bar axis
        caxrect = axrect.copy()
        lax = axrect[0]
        wax = axrect[2]
        margin = wax * .01
        clax = lax + wax + margin
        cwax = (1 - clax) * .3
        caxrect[0] = clax
        caxrect[2] = cwax
        caxrect = cax_kw.pop('rect', caxrect)
        cax = fig.add_axes(caxrect, **cax_kw)
        # Get times as date/time object
        times = self.getTimes()
        # Get duration of sample
        periods = self.variables['PERIOD']

        # add ending bound
        time_edges = np.append(
            times, times[-1] + timedelta(seconds=float(periods[-1])))

        # Get altitude edges
        altedges = self.variables['altitude_edges']
        pr = self.variables['blview_backscatter_profile']
        plot_kw.setdefault('norm', plt.matplotlib.colors.LogNorm())
        patches = ax.pcolormesh(
            time_edges[:], altedges[:], pr[:].T, **plot_kw)
        plt.colorbar(patches, cax=cax, label=pr.units.strip())
        return fig


class test_vaisala(unittest.TestCase):
    def setUp(self):
        from PseudoNetCDF.testcase import ceilometerfiles_paths
        self.vaisalapath = ceilometerfiles_paths['vaisala']

    def testVAISALA2NCF(self):
        import os
        vfile = ceilometerl2(self.vaisalapath)
        outpath = self.vaisalapath + '.nc'
        ncfile = vfile.save(outpath)
        for k, ncv in ncfile.variables.items():
            vpv = vfile.variables[k]
            np.testing.assert_allclose(ncv[...], vpv[...])
        os.remove(outpath)


if __name__ == '__main__':
    from PseudoNetCDF import pncopen
    testdir = '/work/ROMO/users/kpc/flintfx/convceilometer/'
    testpath = testdir + 'CEILOMETER_1_LEVEL_2_20.his'
    f = pncopen(testpath, format='ceilometerl2')



__doc__ = """
.. _vertical_diffusivity
:mod:`vertical_diffusivity` -- Vertical Diffusivity File Interfaces
===================================================================

.. module:: vertical_diffusivity
   :platform: Unix, Windows
   :synopsis: Provides :ref:`PseudoNetCDF` memory map and random access read
   based file interfaces for CAMx vertical diffusivity files.
.. moduleauthor:: Barron Henderson <barronh@unc.edu>
"""
__all__ = ['Memmap', 'Read', 'Transforms', 'Write']

from . import Memmap
from . import Read
from . import Write
from . import Transforms

if __name__ == '__main__':
    vertical_diffusivity = Memmap.vertical_diffusivity
    from PseudoNetCDF.pncdump import pncdump_parser, dump_from_cmd_line
    parser = pncdump_parser()
    parser.add_argument("cols", int)
    parser.add_argument("rows", int)
    (file_path, options, extra_args_dict) = parser.parse_args()

    dump_from_cmd_line(file_path, options,
                       lambda path: vertical_diffusivity(path,
                                                         **extra_args_dict))



#!/usr/bin/env python
import matplotlib.pyplot as plt
from PseudoNetCDF.coordutil import gettimes
import numpy as np


def plotts(args):
    ifiles = args.ifiles
    times = [gettimes(ifile) for ifile in ifiles]
    fig = plt.figure()
    ax = fig.add_subplot(111)  # add_axes([.1, .15, .8, .8])
    ax.set_xlabel('Time (UTC)')
    split = 25
    for target in args.variables:
        vars = [ifile.variables[target] for ifile in ifiles]
        unit = getattr(vars[0], 'units', 'unknown')
        ax.set_ylabel(target + '(' + unit + ')')
        del ax.lines[:]
        nvars = len(vars)
        varwidth = .8 / nvars / 1.1
        po = np.arange(24) + 0.1 + varwidth / 2
        for vi, (time, var) in enumerate(zip(times, vars)):
            vals = var[:]
            if args.squeeze:
                vals = vals.squeeze()
            vardesc = getattr(var, 'description', None)
            varb = ax.plot(time, vals[:], label=vardesc)
        # plt.setp(ax.xaxis.get_ticklabels(),rotation = 45)
        plt.legend()
        figpath = args.outpath + target + '.' + args.figformat
        for pc in args.plotcommands:
            exec(pc, globals(), locals())
        fig.savefig(figpath)
        if args.verbose > 0:
            print('Saved fig', figpath)
        print(figpath)


if __name__ == '__main__':
    from PseudoNetCDF.pncparse import getparser, pncparse
    parser = getparser(plot_options=True, has_ofile=True)
    parser.epilog += """
    -----
box.py inobs inmod target [target ...]
inobs - path to obs
inmod - path to mod
target - variable name
"""
    ifiles, args = pncparse(plot_options=True, has_ofile=True, parser=parser)
    plotts(args)



import importlib
import unittest
from distutils.version import LooseVersion


def _importorskip(modname, minversion=None):
    try:
        mod = importlib.import_module(modname)
        has = True
        if minversion is not None:
            if LooseVersion(mod.__version__) < LooseVersion(minversion):
                raise ImportError('Minimum version not satisfied')
    except ImportError:
        has = False
    # TODO: use pytest.skipif instead of unittest.skipUnless
    # Using `unittest.skipUnless` is a temporary workaround for pytest#568,
    # wherein class decorators stain inherited classes.
    # xref: xarray#1531, implemented in xarray #1557.
    func = unittest.skipUnless(has, reason='requires {}'.format(modname))
    return has, func


has_pyproj, requires_pyproj = _importorskip('pyproj')
has_basemap, requires_basemap = _importorskip('mpl_toolkits.basemap')



import numpy as np
from PseudoNetCDF import PseudoNetCDFFile
from datetime import datetime
from .arltime import arl2time

strptime = datetime.strptime

_packhdrfmt = np.dtype('>i,>S4,>i,>i')

_ijcdt = np.dtype(dict(names='IJC',
                       formats='>i2,>i2,>f'.split(',')))


class arlconcdump(PseudoNetCDFFile):
    @classmethod
    def isMine(cls, *args, **kwds):
        kwds.pop('metaonly', None)
        try:
            cls(*args, metaonly=True, **kwds)
            return True
        except Exception:
            return False

    def __init__(self, path, metaonly=False, vector=False):
        self._infile = open(path, 'rb')
        self._vector = vector
        self._infile.seek(0, 0)
        self._rec12345()
        if not metaonly:
            self._datarec()
        lat = self.createVariable(
            'latitude', 'f', ('latitude',), units='degrees_north',
            values=(self.LLCRNR_LAT + self.DELTA_LAT / 2 +
                    np.arange(self.NLATS) * self.DELTA_LAT)
        )
        lon = self.createVariable(
            'longitude', 'f', ('longitude',), units='degrees_east',
            values=(self.LLCRNR_LON + self.DELTA_LON / 2 +
                    np.arange(self.NLONS) * self.DELTA_LON)
        )
        self.createVariable(
            'latitude_bounds', 'f', ('latitude', 'nv'), units='degrees_north',
            values=np.array([lat - self.DELTA_LAT / 2,
                             lat + self.DELTA_LAT / 2]).T
        )
        self.createVariable(
            'longitude_bounds', 'f', ('longitude', 'nv'), units='degrees_east',
            values=np.array([lon - self.DELTA_LON / 2,
                             lon + self.DELTA_LON / 2]).T
        )
        if self._vector:
            tempf = self.subsetVariables(['latitude', 'latitude_bounds',
                                          'longitude', 'longitude_bounds',
                                          'time', 'time_bounds',
                                          'layer'])\
                        .sliceDimensions(latitude=self.variables['J'],
                                         longitude=self.variables['I'],
                                         time=self.variables['T'],
                                         layer=self.variables['K'])\
                        .renameDimensions(latitude='points',
                                          longitude='points',
                                          time='points',
                                          layer='points')
            for key in list(tempf.variables):
                self.variables[key] = tempf.variables[key]

    def _rec12345(self):
        """
        Record #1

        CHAR*4 Meteorological MODEL Identification
        INT*4 Meteorological file starting time (YEAR, MONTH, DAY, HOUR,
                                                 FORECAST-HOUR)
        INT*4 NUMBER of starting locations
        INT*4 Concentration packing flag (0=no 1=yes)
        """
        infile = self._infile
        rec1 = np.fromfile(
            infile, dtype='>i,>S4,>i,>i,>i,>i,>i,>i,>i,>i', count=1)[0]
        assert(rec1['f0'] == rec1['f9'])
        nloc = self.NSTARTLOCS = rec1['f7']
        rec1['f8'] == 1
        self.METMODEL = rec1['f1'].decode()
        self.MET_YEAR = rec1['f2']
        self.MET_MONTH = rec1['f3']
        self.MET_DAY = rec1['f4']
        self.MET_HOUR = rec1['f5']
        self.MET_FORECAST_HOUR = rec1['f6']
        self.STARTING_LOCATIONS = rec1['f7']
        self.PACKED = rec1['f8']

        """
        Record #2 Loop to record: Number of starting locations

        INT*4 Release starting time (YEAR, MONTH, DAY, HOUR)
        REAL*4 Starting location and height (LATITUDE, LONGITUDE, METERS)
        INT*4 Release starting time (MINUTES)
        """
        rec2 = np.fromfile(infile, dtype='>i,>4i,>3f,>i,>i', count=nloc)
        assert((rec2['f0'] == rec2['f4']).all())
        self.createDimension('starts', nloc)
        sy = self.createVariable('START_YEAR', 'i', ('starts',), units='year')
        sy[:] = rec2['f1'][:, 0]
        sm = self.createVariable(
            'START_MONTH', 'i', ('starts',), units='month')
        sm[:] = rec2['f1'][:, 1]
        sd = self.createVariable('START_DAY', 'i', ('starts',), units='day')
        sd[:] = rec2['f1'][:, 2]
        sh = self.createVariable('START_HOUR', 'i', ('starts',), units='hour')
        sh[:] = rec2['f1'][:, 3]
        slat = self.createVariable(
            'START_LAT', 'f', ('starts',), units='degrees_north')
        slat[:] = rec2['f2'][:, 0]
        slon = self.createVariable(
            'START_LON', 'f', ('starts',), units='degrees_east')
        slon[:] = rec2['f2'][:, 1]
        salt = self.createVariable(
            'START_ALT', 'f', ('starts',), units='meters')
        salt[:] = rec2['f2'][:, 2]

        """
        Record #3

        INT*4 Number of (LATITUDE-POINTS, LONGITUDE-POINTS)
        REAL*4 Grid spacing (DELTA-LATITUDE,DELTA-LONGITUDE)
        REAL*4 Grid lower left corner (LATITUDE, LONGITUDE)
        """
        rec3 = np.fromfile(infile, dtype='>i,>2i,>2f,>2f,>i', count=1)[0]
        assert(rec3['f0'] == rec3['f4'])

        self.NLATS = rec3['f1'][0]
        self.NLONS = rec3['f1'][1]
        self.DELTA_LAT = rec3['f2'][0]
        self.DELTA_LON = rec3['f2'][1]
        self.LLCRNR_LAT = rec3['f3'][0]
        self.LLCRNR_LON = rec3['f3'][1]
        """
        Record #4

        INT*4 NUMBER of vertical levels in concentration grid
        INT*4 HEIGHT of each level (meters above ground)
        """
        tmp = np.fromfile(infile, dtype='>i,>i', count=1)[0]
        nlays = self.NLAYS = tmp['f1']
        infile.seek(-8, 1)
        rec4 = np.fromfile(
            infile, dtype='>i,>i,>{}i,>i'.format(nlays), count=1)[0]
        assert(rec4['f0'] == rec4['f3'])
        self.createDimension('layer', nlays)
        var = self.createVariable('layer', 'i', ('layer',))
        var.units = 'meters agl'
        var[:] = rec4['f2']

        """
        Record #5

        INT*4 NUMBER of different pollutants in grid
        CHAR*4 Identification STRING for each pollutant
        """
        tmp = np.fromfile(infile, dtype='>i,>i', count=1)
        npols = self.NPOLS = tmp['f1'][0]
        if npols > 1 and self._vector:
            raise IOError('The vector option and multipollutant files ' +
                          'are not compatible')
        infile.seek(-8, 1)
        rec5 = np.fromfile(infile, dtype='>i,>i,>({},)S4,>i'.format(
            npols), count=1).squeeze()
        assert(rec5['f0'] == rec5['f3'])
        self.POLLUTANTS = b','.join(rec5['f2']).decode()

    def _datarec(self):
        infile = self._infile
        nlats = self.NLATS
        nlons = self.NLONS
        nlays = self.NLAYS
        npols = self.NPOLS
        pack = self.PACKED == 1
        now = infile.tell()
        infile.seek(0, 2)
        total = infile.tell()
        infile.seek(now, 0)
        datasize = total - now
        datasize
        thdr = np.dtype('>i,>6i,>i')
        nopackdfmt = '>({},{})f'.format(nlats, nlons)
        nopackfmt = np.dtype(
            dict(
                names=['B', 'POL', 'LAY', 'data', 'E'],
                formats=['>i', '>S4', '>i', nopackdfmt, '>i']
            )
        )
        outs = []
        pols = []
        lays = []
        starts = []
        stops = []
        ti = 0
        while infile.tell() != total:
            """
            Record #6 Loop to record: Number of output times

            INT*4 Sample start (YEAR MONTH DAY HOUR MINUTE FORECAST)
            """
            start = np.fromfile(infile, dtype=thdr, count=1)
            starts.append(start['f1'][0])
            """
            Record #7 Loop to record: Number of output times

            INT*4 Sample stop (YEAR MONTH DAY HOUR MINUTE FORECAST)
            """
            stop = np.fromfile(infile, dtype=thdr, count=1)
            stops.append(stop['f1'][0])
            """
            Record #8 Loop to record: Number levels, Number of pollutant types

            CHAR*4 Pollutant type identification STRING
            INT*4 Output LEVEL (meters) of this record

            No Packing (all elements)
            REAL*4 Concentration output ARRAY

            Packing (only non-zero elements)

            INT*4 Loop non-zero elements
            INT*2 First (I) index value
            INT*2 - Second (J) index value
            REAL*4 - Concentration at (I,J)**_
            """
            if pack:
                # c = np.zeros((npols, nlays, nlats, nlons), dtype='f')
                for pi in range(npols):
                    for li in range(nlays):
                        tmp = np.fromfile(infile, _packhdrfmt, count=1)
                        myp = tmp['f1'][0]
                        myl = tmp['f2'][0]
                        pols.append(myp)
                        lays.append(myl)
                        myc = tmp['f3'][0]
                        tmp = np.fromfile(infile,
                                          np.dtype([('data', _ijcdt, myc),
                                                    ('f1', '>i', 1)]), count=1)
                        tdata = tmp['data'][0]
                        Jidx = tdata['J'] - 1
                        Iidx = tdata['I'] - 1

                        # c[pi, li, Jidx, Iidx] = tdata['C']
                        if self._vector:
                            if myc > 0:
                                zl = np.array(np.ones_like(Jidx), ndmin=1)
                                outs.append([
                                    ti * zl, pi * zl, li * zl, Jidx,
                                    Iidx, tdata['C']]
                                )
                        else:
                            outs.append([ti, pi, li, Jidx, Iidx, tdata['C']])
            else:
                npblock = np.fromfile(
                    infile, dtype=nopackfmt, count=npols * nlays)
                c = npblock['data']
                outs.append(c.reshape(npols, nlays, nlats, nlons))
            ti += 1

        ntimes = ti
        if isinstance(outs[0], list):
            if self._vector:
                datablock = np.concatenate(
                    [
                        np.array(out, ndmin=2, dtype='f').reshape(6, -1)
                        for out in outs
                    ],
                    axis=1
                ).T
                npoints = datablock.shape[0]
            else:
                datablock = np.zeros((ntimes, npols, nlays, nlats, nlons),
                                     dtype='f')
                for ti, pi, li, ji, ii, v in outs:
                    datablock[ti, pi, li, ji, ii] = v
        else:
            datablock = np.stack(outs, axis=0)
        pols = np.array(pols).reshape(ntimes, npols, nlays)
        lays = np.array(lays).reshape(ntimes, npols, nlays)
        self.createDimension('time', ntimes)
        if self._vector:
            self.createDimension('points', npoints)
        self.createDimension('layer', nlays)
        self.createDimension('latitude', nlats)
        self.createDimension('longitude', nlons)
        self.createDimension('nv', 2)
        assert((lays[:, [0], :] == lays[:, :, :]).all())
        assert((pols[:, :, [0]] == pols[:, :, :]).all())
        if self._vector:
            poldims = ('points',)
            for key, idx in [('T', 0), ('J', -3), ('I', -2), ('K', -4)]:
                self.createVariable(
                    key, 'i', poldims,
                    units='0-based index',
                    values=datablock[:, idx].astype('i')
                )
        else:
            poldims = ('time', 'layer', 'latitude', 'longitude')
        for pi, pol in enumerate(pols[0, :, 0]):
            if self._vector:
                poldata = datablock[:, -1][datablock[:, 1] == pi]
            else:
                poldata = datablock[:, pi]
            var = self.createVariable(pol.decode(), 'f', poldims,
                                      values=poldata)
            var.units = 'arbitrary'
            var.description = pol.decode()

        time = self.createVariable(
            'time', 'd', ('time',),
            units='seconds since 1970-01-01 00:00:00+0000'
        )
        time_bounds = self.createVariable(
            'time_bounds', 'd', ('time', 'nv'),
            units='seconds since 1970-01-01 00:00:00+0000'
        )
        rdate = strptime('1970-01-01 00:00:00+0000', '%Y-%m-%d %H:%M:%S%z')
        sdt = np.array([(arl2time(YY, MM, DD, HH, mm) - rdate).total_seconds()
                       for YY, MM, DD, HH, mm, FF in starts])
        edt = np.array([(arl2time(YY, MM, DD, HH, mm) - rdate).total_seconds()
                       for YY, MM, DD, HH, mm, FF in stops])
        time_bounds[:, 0] = sdt
        time_bounds[:, 1] = edt
        time[:] = time_bounds.mean(1)


if __name__ == '__main__':
    print(arlconcdump.isMine('cdump24'))
    f = arlconcdump('cdump24')



from __future__ import print_function
__all__ = ['osat']
__doc__ = """
.. _Write
:mod:`Write` -- CAMx uamiv variable transformations
==================================================

.. module:: Write
   :platform: Unix, Windows
   :synopsis: Provides :ref:`PseudoNetCDF` variable transformations
              for CAMx uamiv files.  See PseudoNetCDF.sci_var.PseudoNetCDFFile
              for interface details
.. moduleauthor:: Barron Henderson <barronh@unc.edu>
"""

from numpy import zeros
from PseudoNetCDF.sci_var import PseudoNetCDFFile, PseudoNetCDFVariables
from PseudoNetCDF.sci_var import PseudoIOAPIVariable
from .Memmap import uamiv


class osat(PseudoNetCDFFile):
    """
    OSAT provides a way of slicing and summing tagged species.

    sources - dictionary of name and id for CAMx source codes
              (i.e. {'SOURCES1-5': '001','002','003','004','005'})
    regions - dictionary of name and id for CAMx region codes
              (i.e. {'REGIONS1-5': '001','002','003','004','005'})

    Example:
        # Source and regions definitions
        sources={'SOURCES1-5': ('001','002','003','004','005')}
        regions={'REGIONS1-5': ('001','002','003','004','005')}
        # File initiation
        osatfile=osat('path',sources,regions)
        # Variable (v) would return the sum of all O3V for all five
        # sources and all five regions
        v=osatfile.variables['O3V_SOURCES1-5_REGIONS1-5']

    """
    __delim = '_'

    def __init__(self, rffile, sources={}, regions={}):
        self.__child = uamiv(rffile)

        # Use Use all species based keys
        self.__sourcesbyNm = dict(
            [(k[3:6], k[3:6])
             for k in self.__child.variables.keys()
             if k[:3] in ('NOX', 'VOC', 'O3V', 'O3N')])
        self.__regionsbyNm = dict(
            [(k[6:], k[6:])
             for k in self.__child.variables.keys()
             if k[:3] in ('NOX', 'VOC', 'O3V', 'O3N')])

        # Create global keys
        self.__sourcesbyNm[''] = tuple(self.__sourcesbyNm.keys())
        self.__regionsbyNm[''] = tuple(self.__regionsbyNm.keys())

        for k, v in sources.items():
            if type(v) == str:
                sources[k] = (v,)
        for k, v in regions.items():
            if type(v) == str:
                regions[k] = (v,)

        # Update with user supplied keys
        self.__sourcesbyNm.update(sources)
        self.__regionsbyNm.update(regions)

        self.dimensions = self.__child.dimensions.copy()

        spc_keys = ['NOX', 'VOC', 'O3N', 'O3V']
        allkeys = [i for i in self.__child.variables.keys()]
        for skey in spc_keys:
            for src in sources.keys() + ['']:
                for reg in regions.keys() + ['']:
                    allkeys.append(self.__delim.join([skey, src, reg]))
        allkeys = [i for i in set(allkeys)]
        self.variables = PseudoNetCDFVariables(self.__variables, allkeys)

    def __indices(self, keys):
        return [self.__child.__var_names__.index(k) for k in keys]

    def __var_id_names(self, var_name):
        components = var_name.split(self.__delim)
        if len(components) == 1:
            keys = components
        else:
            spc, source, region = components
            keys = []
            sources = self.__sourcesbyNm[source]
            regions = self.__regionsbyNm[region]
            for source in sources:
                for region in regions:
                    key = spc + source + region
                    if key in self.__child.variables.keys():
                        keys.append(key)
        return keys

    def __variables(self, key):
        var_id_names = self.__var_id_names(key)
        # var_nm_names=self.__var_nm_names(key)

        if len(var_id_names) > 1:
            outvals = zeros([len(self.dimensions[dk])
                             for dk in ['TSTEP', 'LAY', 'ROW', 'COL']],
                            dtype='>f')
            for k in var_id_names:
                outvals[...] += self.__child.variables[k]
        else:
            outvals = self.__child.variables[var_id_names[0]]

        dimensions = ('TSTEP', 'VAR', 'LAY', 'ROW', 'COL')
        v = PseudoIOAPIVariable(
            self, key, 'f', dimensions, values=outvals, units='ppm')
        v.VAR_NAMES = ''.join([nm.ljust(16) for nm in var_id_names])
        # v.VAR_NAME_DESCS=''.join([nm.ljust(16) for nm in var_nm_names])
        return v



__all__ = ['height_pressure_plus',
           'height_pressure_center_time_plus', 'height_pressure_center_time']
__doc__ = """
.. _Write
:mod:`Write` -- CAMx height/pressure variable transformations
=============================================================

.. module:: Write
   :platform: Unix, Windows
   :synopsis: Provides :ref:`PseudoNetCDF` variable transformations
              for CAMx height/pressure files.  See PseudoNetCDFFile
              for interface details
.. moduleauthor:: Barron Henderson <barronh@unc.edu>
"""

from PseudoNetCDF.MetaNetCDF import add_derived, time_avg_new_unit
from PseudoNetCDF.sci_var import PseudoNetCDFVariable
from PseudoNetCDF.camxfiles.height_pressure.Memmap import height_pressure
from PseudoNetCDF.ArrayTransforms import CAMxHeightToDepth
reg_height_pressure = height_pressure


class height_pressure_plus(add_derived):
    __childclass__ = reg_height_pressure
    __addvars__ = ['DEPTH']

    def __DEPTH__(self):
        val = CAMxHeightToDepth(self.variables['HGHT'])
        var = PseudoNetCDFVariable(
            self, 'DEPTH', 'f', ('TSTEP', 'LAY', 'ROW', 'COL'), values=val)
        var.units = 'm'
        var.long_name = 'RATE'.ljust(16)
        var.var_desc = 'RATE'.ljust(16)
        return var


class height_pressure_center_time_plus(time_avg_new_unit):
    __reader__ = height_pressure_plus


class height_pressure_center_time(time_avg_new_unit):
    __reader__ = reg_height_pressure



#!/usr/bin/env python
import matplotlib.pyplot as plt
import numpy as np


def plot_scatter(ifiles, args):
    fig = plt.figure()
    if len(args.figure_keywords) > 0:
        plt.setp(fig, **args.figure_keywords)
    sax = fig.add_subplot(111)  # add_axes([.1, .15, .8, .8])
    if len(args.axes_keywords) > 0:
        plt.setp(sax, **args.axes_keywords)
    ifilex = ifiles[0]
    ifiley = ifiles[1]
    for target in args.variables:
        varx = ifilex.variables[target]
        varxdesc = getattr(varx, 'description', None)
        vary = ifiley.variables[target]
        varydesc = getattr(vary, 'description', None)
        sax.set_ylabel(varydesc)
        sax.set_xlabel(varxdesc)
        valx = np.ma.compressed(varx[:])
        valy = np.ma.compressed(vary[:])
        if valx.size < valy.size:
            svalx = np.sort(valx)
            svaly = np.percentile(valy, np.arange(
                svalx.size, dtype='f') / (svalx.size - 1) * 100)
        elif valy.size < valx.size:
            svaly = np.sort(valy)
            pctidx = np.arange(svaly.size, dtype='f') / (svaly.size - 1) * 100
            svalx = np.percentile(valx, pctidx)
        else:
            svaly = np.sort(valy)
            svalx = np.sort(valx)

        del sax.lines[:]
        vmin = np.minimum(varx[:].min(), vary[:].min())
        vmax = np.maximum(varx[:].max(), vary[:].max())
        sax.plot([vmin, vmax], [vmin, vmax], color='k')
        sax.plot(svalx[:], svaly[:], ls='none',
                 marker='o', markeredgecolor='none')
        sax.set_xlim(vmin, vmax)
        sax.set_ylim(vmin, vmax)
        figpath = args.outpath + target + '.' + args.figformat
        fig.savefig(figpath)
        if args.verbose > 0:
            print('Saved fig', figpath)


if __name__ == '__main__':
    from PseudoNetCDF.pncparse import getparser, pncparse
    parser = getparser(plot_options=True, has_ofile=True)
    parser.epilog += """
    -----
box.py inobs inmod target [target ...]
inobs - path to obs
inmod - path to mod
target - variable name
"""
    ifiles, args = pncparse(plot_options=True, has_ofile=True, parser=parser)
    plot_scatter(ifiles, args)



from __future__ import print_function, unicode_literals
__all__ = ['ncf2uamiv', 'write_emissions_ncf', 'write_emissions']
__doc__ = """
.. _Write
:mod:`Write` -- CAMx uamiv  writer
============================================

.. module:: Write
   :platform: Unix, Windows
   :synopsis: Provides :ref:`PseudoNetCDF` writer for CAMx
              uamiv files.  See PseudoNetCDF.sci_var.PseudoNetCDFFile
              for interface details
.. moduleauthor:: Barron Henderson <barronh@unc.edu>
"""

# Distribution packages
import unittest
import sys
import os
import operator
from functools import reduce

# Site-Packages
import numpy as np

# This Package modules
from PseudoNetCDF.camxfiles.timetuple import timeadd, timerange
from PseudoNetCDF.camxfiles.FortranFileUtil import writeline, Asc2Int
from PseudoNetCDF._getwriter import registerwriter

_emiss_hdr_fmt = np.dtype(dict(
    names=['SPAD', 'name', 'note', 'itzon', 'nspec', 'ibdate', 'btime',
           'iedate', 'etime', 'EPAD'],
    formats=['>i', '(10,4)>S1', '(60,4)>S1', '>i', '>i', '>i', '>f', '>i',
             '>f', '>i']))

_grid_hdr_fmt = np.dtype(dict(
    names=['SPAD', 'plon', 'plat', 'iutm', 'xorg', 'yorg', 'delx', 'dely',
           'nx', 'ny', 'nz', 'iproj', 'istag', 'tlat1',
           'tlat2', 'rdum5', 'EPAD'],
    formats=['>i', '>f', '>f', '>i', '>f', '>f', '>f', '>f', '>i', '>i', '>i',
             '>i', '>i', '>f', '>f', '>f', '>i']))

_cell_hdr_fmt = np.dtype(dict(
    names=['SPAD', 'ione1', 'ione2', 'nx', 'ny', 'EPAD'],
    formats=['>i', '>i', '>i', '>i', '>i', '>i']))

_time_hdr_fmt = np.dtype(dict(
    names=['SPAD', 'ibdate', 'btime', 'iedate', 'etime', 'EPAD'],
    formats=['>i', '>i', '>f', '>i', '>f', '>i']))

_spc_fmt = np.dtype("(10,4)>S1")


def ncf2uamiv(ncffile, outpath):
    """
    ncf2uamiv converts a ncffile to a uamiv file

    Parameters
    ----------
    ncffile : PseudoNetCDF-like object
            Must be open and have all properties of a uamiv
            file as produced by PseudoNetCDF.camxfiles
            1) IOAPI conventions
            2) UAMIV specific properties: PLON PLAT IUTM CPROJ
                                            TLAT1 TLAT2 ISTAG
    outpath : string
            path to create a uamiv file output

    Returns
    -------
    outfile : file object
            File object is in write binary mode and has the
            uamiv file as its contents

    Examples
    --------
        $ # test.uamiv must be a file in uamiv format
        $ pncgen -f uamiv test.uamiv test.uamiv.nc
        $ python -c "
        ncpath = 'test.uamiv.nc'
        uamivpath = 'test.uamiv.nc.uamiv'
        from netCDF4 import Dataset
        from PseudoNetCDF.camxfiles.uamiv.Write import ncf2uamiv
        inncf = Dataset(ncpath)
        ncf2uamiv(inncf, uamivpath)
        "
        # If uamivpath does not include EOD, the diff will be perfect
        diff test.uamiv test.uamiv.nc.uamiv

        # If uamivpath includes EOD, the diff may yield difference.
        # The ONLY difference will be time flags for the end of a day.
        # Most time software agrees -- there is no such thing as 2002154T24:00.
        # Some CAMx files, however, have a 2400 time.
        # PseudoNetCDF interprets this equivalently as 2002155T00:00
        $ python -c "from PseudoNetCDF.camxfiles.Memmaps import uamiv
        import numpy as np
        old = uamiv('test.uamiv')
        new = uamiv('test.uamiv.nc.uamiv')
        for k in old.variables.keys():
            check = (old.variables[k][...] == new.variables[k][...])
            if not check.all():
                print(k)
                if k == 'ETFLAG':
                    diffidx = np.where(~check)[:2]
                else:
                    diffidx = np.where(~check)
                print(old.variables[k][diffidx])
                print(new.variables[k][diffidx])

    """

    emiss_hdr = np.zeros(shape=(1,), dtype=_emiss_hdr_fmt)
    emiss_hdr[0]['name'][:, :] = ' '
    emiss_hdr[0]['name'][:, 0] = np.array(ncffile.NAME, dtype='>c')
    emiss_hdr[0]['note'][:, :] = ' '
    emiss_hdr[0]['note'][:, 0] = np.array(ncffile.NOTE, dtype='>c')
    # gdtype = getattr(ncffile, 'GDTYP', -999)
    emiss_hdr['itzon'][0] = ncffile.ITZON
    nspec = len(ncffile.dimensions['VAR'])
    emiss_hdr['nspec'] = nspec

    NCOLS = len(ncffile.dimensions['COL'])
    NROWS = len(ncffile.dimensions['ROW'])
    NLAYS = len(ncffile.dimensions['LAY'])
    grid_hdr = np.zeros(shape=(1,), dtype=_grid_hdr_fmt)
    grid_hdr['SPAD'] = grid_hdr.itemsize - 8
    grid_hdr['plon'] = ncffile.PLON
    grid_hdr['plat'] = ncffile.PLAT
    grid_hdr['iutm'][0] = ncffile.IUTM
    grid_hdr['xorg'] = ncffile.XORIG
    grid_hdr['yorg'] = ncffile.YORIG
    grid_hdr['delx'] = ncffile.XCELL
    grid_hdr['dely'] = ncffile.YCELL
    grid_hdr['nx'] = NCOLS
    grid_hdr['ny'] = NROWS
    grid_hdr['nz'] = NLAYS
    grid_hdr['iproj'] = ncffile.CPROJ
    grid_hdr['tlat1'] = ncffile.TLAT1
    grid_hdr['tlat2'] = ncffile.TLAT2
    grid_hdr['istag'] = ncffile.ISTAG
    grid_hdr['rdum5'] = 0.
    grid_hdr['EPAD'] = grid_hdr.itemsize - 8

    cell_hdr = np.zeros(shape=(1,), dtype=_cell_hdr_fmt)
    cell_hdr['SPAD'] = cell_hdr.itemsize - 8
    cell_hdr['ione1'] = 1
    cell_hdr['ione2'] = 1
    cell_hdr['nx'] = NCOLS
    cell_hdr['ny'] = NROWS
    cell_hdr['EPAD'] = cell_hdr.itemsize - 8

    time_hdr = np.zeros(
        shape=(len(ncffile.dimensions['TSTEP']),), dtype=_time_hdr_fmt)
    time_hdr['SPAD'] = 16
    time_hdr['EPAD'] = 16
    date_s, time_s = ncffile.variables['TFLAG'][:, 0].T
    time_s = time_s.astype('>f') / 10000.
    date_s = date_s % (date_s // 100000 * 100000)
    if 'ETFLAG' in ncffile.variables.keys():
        date_e, time_e = ncffile.variables['ETFLAG'][:, 0].T
        time_e = time_e.astype('>f') / 10000.
        date_e = date_e % (date_e // 100000 * 100000)
    else:
        if hasattr(ncffile, 'TSTEP'):
            tincr = ncffile.TSTEP / 10000
        else:
            tincr = np.diff(time_s)[0]
        date_e = date_s.copy()
        time_e = time_s.copy() + tincr
        date_e += (time_e // 24).astype('i')
        time_e -= (time_e // 24) * 24
    time_hdr['ibdate'] = date_s
    time_hdr['btime'] = time_s
    time_hdr['iedate'] = date_e
    time_hdr['etime'] = time_e
    emiss_hdr['ibdate'] = date_s[0]
    emiss_hdr['btime'] = time_s[0]
    emiss_hdr['iedate'] = date_e[-1]
    emiss_hdr['etime'] = time_e[-1]
    emiss_hdr['SPAD'] = _emiss_hdr_fmt.itemsize - 8
    emiss_hdr['EPAD'] = _emiss_hdr_fmt.itemsize - 8

    spc_hdr = np.zeros(shape=(1,), dtype=dict(
        names=['SPAD1', 'DATA', 'EPAD1'],
        formats=['>i', np.dtype("(%d,10,4)>S1" % nspec), '>i']))
    spc_hdr['SPAD1'] = nspec * 40
    spc_hdr['EPAD1'] = nspec * 40
    spc_names = np.array(getattr(ncffile, 'VAR-LIST'),
                         dtype='>c').reshape(-1, 16)[:, :10].copy()
    spc_hdr[0]['DATA'][:] = ' '
    spc_hdr[0]['DATA'][:, :, 0] = spc_names
    spc_names = [s.decode() if hasattr(s, 'decode')
                 else s for s in spc_names.view('>S10')[:, 0]]
    nz = len(ncffile.dimensions['LAY'])
    outfile = open(outpath, 'wb')
    emiss_hdr.tofile(outfile)
    grid_hdr.tofile(outfile)
    cell_hdr.tofile(outfile)
    spc_hdr.tofile(outfile)
    for di, (d, t) in enumerate(ncffile.variables['TFLAG'][:, 0]):
        time_hdr[di].tofile(outfile)
        for spc_key, spc_name in zip(spc_names, spc_hdr[0]['DATA']):
            for zi in range(nz):
                var = ncffile.variables[str(np.char.strip(spc_key))]
                data = var[di, zi].astype('>f')
                buf = np.array(4 + 40 + data.size * 4).astype('>i')
                buf.tofile(outfile)
                np.array(1).astype('>i').tofile(outfile)
                spc_name.tofile(outfile)
                np.ma.filled(data).tofile(outfile)
                buf.tofile(outfile)
    outfile.flush()
    return outfile


registerwriter('camxfiles.uamiv', ncf2uamiv)
registerwriter('uamiv', ncf2uamiv)


def write_emissions_ncf(infile, outfile):
    from operator import concat
    # initialize hdr_fmts with species count
    hdr_fmts = ["10i60i3ifif", "ffiffffiiiiifff",
                "iiii", "10i" * len(infile.variables.keys())]
    hdrlines = []

    hdrlines.append(
        reduce(concat, [Asc2Int(s) for s in [infile.name, infile.note]]) +
        [infile.ione, len(infile.variables.keys()), infile.start_date,
         infile.start_time, infile.end_date, infile.end_time])

    hdrlines.append(
        [infile.rdum, infile.rdum, infile.iutm, infile.xorg, infile.yorg,
         infile.delx, infile.dely, len(infile.dimensions['COL']),
         len(infile.dimensions['ROW']), len(infile.dimensions['LAY']),
         infile.idum, infile.idum, infile.rdum, infile.rdum, infile.rdum])

    hdrlines.append([infile.ione, infile.ione, len(
        infile.dimensions['COL']), len(infile.dimensions['ROW'])])
    hdrlines.append(reduce(concat, [Asc2Int(s.ljust(10))
                                    for s in infile.variables.keys()]))

    for d, h in zip(hdrlines, hdr_fmts):
        outfile.write(writeline(d, h))

    for ti, (d, t) in enumerate(infile.timerange()):
        ed, et = timeadd((d, t), (0, infile.time_step))
        outfile.write(writeline((d, t, ed, et), 'ifif'))
        for spc in infile.variables.keys():
            var = infile.variables[spc]
            for k in range(len(infile.dimensions['LAY'])):
                outfile.write(writeline(
                    [infile.ione] +
                    Asc2Int(spc.ljust(10)) +
                    var[ti, :, :, k].transpose().ravel().tolist(),
                    '11i' + infile.cell_count * 'f'))


def write_emissions(start_date, start_time, time_step, hdr, vals):
    # initialize hdr_fmts with species count
    hdr_fmts = ["10i60i3ifif", "ffiffffiiiiifff", "iiii", "10i" * len(hdr[-1])]

    # initialize output variable
    emis_string = ''

    # Internalize header
    hdr = [h for h in hdr]
    species = hdr[-1]

    # Check start_date and start_time
    if tuple(hdr[0][4:6]) != (start_date, start_time):
        print("Header doesn't match start date/time", file=sys.stderr)

    # Change name and note
    hdr[0] = list(hdr[0][0]) + list(hdr[0][1]) + list(hdr[0][2:])

    # Change species names to array of characters
    hdr[-1] = reduce(operator.concat, [Asc2Int(s) for s in hdr[-1]])

    # for each item in the header, write it to output
    for h, f in zip(hdr, hdr_fmts):
        emis_string += writeline(h, f)

    # create value format
    cells = vals.shape[2] * vals.shape[3]
    valfmt = 'i10i' + ('f' * cells)

    # Get end date
    (end_date, end_time) = timeadd(
        (start_date, start_time), (0, time_step * vals.shape[0]))

    if tuple(hdr[0][6:]) != (end_date, end_time):
        print("Header doesn't match end date/time", file=sys.stderr)

    # Write out values
    for ti, (d, t) in enumerate(timerange((start_date, start_time),
                                          (end_date, end_time),
                                          time_step)):
        ed, et = timeadd((d, t), (0, time_step))
        emis_string += writeline((d, t, ed, et), 'ifif')
        for si, spc in enumerate(species):
            for k in range(vals.shape[-1]):
                # Dummy variable,spc characters and values flattened
                temp = [1]
                temp.extend(Asc2Int(spc))
                spcvals = vals[ti, si, ..., ..., k]
                spcvals = spcvals.transpose().ravel()
                temp.extend(spcvals)
                emis_string += writeline(temp, valfmt)

    return emis_string


class TestMemmaps(unittest.TestCase):
    def setUp(self):
        from PseudoNetCDF.testcase import camxfiles_paths
        self.uamivpath = camxfiles_paths['uamiv']

    def testNCF2UAMIV(self):
        from PseudoNetCDF.camxfiles.Memmaps import uamiv
        uamivfile = uamiv(self.uamivpath)
        from PseudoNetCDF.pncgen import pncgen
        pncgen(uamivfile, self.uamivpath + '.check', inmode='r',
               outmode='w', format='uamiv', verbose=0)
        check = True
        uamivfile2 = uamiv(self.uamivpath + '.check')
        for k, v in uamivfile.variables.items():
            nv = uamivfile2.variables[k]
            check = check & bool((nv[...] == v[...]).all())
        assert(check)
        os.remove(self.uamivpath + '.check')



from ..sci_var import PseudoNetCDFFile
from numpy import dtype, fromstring, newaxis, vectorize, nan
from datetime import timedelta, datetime


def box_model_irr(irr_path, start_datetime):
    retval = box_model_conc(irr_path, start_datetime)
    for spc, var in retval.variables.items():
        if spc not in ('TFLAG',):
            var.units = '%s/h' % (var.units.strip(),)

    return retval


def box_model_conc(conc_path, start_datetime):
    lines = open(conc_path, 'r').readlines()[3:]

    spc_names = [spc_name.strip() for spc_name in lines[0][:-1].split('\t')]
    data_block = ''.join(lines[1:])

    line_format = dtype(dict(names=spc_names, formats=['f'] * len(spc_names)))
    conc = fromstring(data_block, dtype='f', sep='\t').view(line_format)

    result = PseudoNetCDFFile()
    result.SDATE = int(start_datetime.strftime('%Y%j'))
    result.STIME = int(start_datetime.strftime('%H%M%S'))
    result.createDimension('TSTEP', conc.shape[0])
    result.createDimension('VAR', 1)
    result.createDimension('DATE-TIME', 2)
    for spc in spc_names:
        var = result.createVariable(spc, 'f', ('TSTEP',))
        var[:] = conc[spc]
        var.long_name = spc.ljust(16)
        var.var_desc = spc.ljust(16)
        var.units = 'ppb'

    tflag = result.createVariable('TFLAG', 'i', ('TSTEP', 'VAR', 'DATE-TIME'))
    start_date = datetime.strptime(start_datetime.strftime('%Y%j'), '%Y%j')
    t_tflag = start_date + \
        vectorize(lambda h: timedelta(hours=float(h)))(
            conc[spc_names[0]].astype('d'))
    tflag[:, :, 0] = vectorize(lambda d: int(
        d.strftime('%Y%j')))(t_tflag[:, newaxis])
    tflag[:, :, 1] = vectorize(lambda d: int(
        d.strftime('%H%M%S')))(t_tflag[:, newaxis])
    tflag.units = "<YYYYMMDD, HHMMSS>"
    tflag.long_name = 'TFLAG'.ljust(16)
    tflag.var_desc = 'TFLAG'.ljust(16)

    return result


def box_model_mrg(conc_path, irr_path, start_datetime):
    retval = PseudoNetCDFFile()
    conc = box_model_conc(conc_path, start_datetime)
    irr = box_model_irr(irr_path, start_datetime)
    sorted_rxns = [k for k in irr.variables.keys()
                   if k not in ('TFLAG', 'TIME')]
    sorted_rxns.sort()
    sorted_spcs = [k for k in conc.variables.keys()
                   if k not in ('TFLAG', 'TIME')]
    sorted_spcs.sort()

    nsteps = retval.createDimension('TSTEP', len(irr.dimensions['TSTEP']) - 1)
    retval.createDimension('REACTIONS', len(sorted_rxns))
    nsteps = len(retval.dimensions['TSTEP'])
    retval.createDimension('SPECIES', len(sorted_spcs))
    retval.createDimension('PROCESS', 3)
    irr_vals = retval.createVariable('IRR', 'f', ('TSTEP', 'REACTIONS'))
    irr_vals.units = 'ppb/h'
    irr_vals.long_name = 'IRR'.ljust(16)
    irr_vals.var_desc = 'IRR'.ljust(16)
    ipr_vals = retval.createVariable(
        'IPR', 'f', ('TSTEP', 'SPECIES', 'PROCESS'))
    ipr_vals.units = 'ppb/h'
    ipr_vals.long_name = 'IPR'.ljust(16)
    ipr_vals.var_desc = 'IPR'.ljust(16)

    for rxni, rxn_name in enumerate(sorted_rxns):
        vals = irr.variables[rxn_name][:]
        vals = vals.repeat(2, 0)[1:-1].reshape(nsteps, 2).mean(1)

        irr_vals[:, rxni] = vals

    irr_vals[0, :] = nan
    retval.Reactions = ''.join([k.ljust(16) for k in sorted_rxns])

    for spci, spc_name in enumerate(sorted_spcs):
        ipr_vals[:, spci, 0] = conc.variables[spc_name][:-1]
        ipr_vals[:, spci, 2] = conc.variables[spc_name][1:]
        ipr_vals[:, spci, 1] = ipr_vals[:, spci, 2] - ipr_vals[:, spci, 0]

    retval.Species = ''.join([k.ljust(16) for k in sorted_spcs])
    retval.Process = 'INIT'.ljust(16) + 'CHEM'.ljust(16) + 'FCONC'.ljust(16)

    return retval



from __future__ import print_function
import sys
import unittest
from PseudoNetCDF.sci_var import PseudoNetCDFFile
from PseudoNetCDF.sci_var import PseudoNetCDFMaskedVariable
from PseudoNetCDF._getwriter import registerwriter
from numpy import vectorize, ndarray, array, genfromtxt
from numpy.ma import MaskedArray, filled
import numpy as np
from datetime import datetime, timedelta
import re
try:
    from StringIO import StringIO
except ImportError:
    from io import BytesIO as StringIO
from warnings import warn

if (sys.version_info > (3, 0)):
    openf = open
else:
    def openf(path, mode, encoding):
        return open(path, mode)

PseudoNetCDFVariable = PseudoNetCDFMaskedVariable


def get_lodval(v):
    try:
        return eval(v)
    except Exception:
        return v


loddelim = re.compile('(;\s?)|,|\s')

PI_LINE = 2
ORG_LINE = 3
PLAT_LINE = 4
MISSION_LINE = 5
VOL_LINE = 6
DATE_LINE = 7
TIME_INT_LINE = 8
UNIT_LINE = 9
DATE_VAR_LINE = 10
SCALE_LINE = 11
MISSING_LINE = 12


class ffi1001(PseudoNetCDFFile):
    """
Overview:
  ffi1001 is a reader object for the NASA AMES format also
  known as the ICARTT file format. The format is defined in detail
  at https://www-air.larc.nasa.gov/missions/etc/IcarttDataFormat.htm
  Standard of practice is to write files in UTF-8 encoding.
  However, it is not uncommon to receive files with special
  characters. To specify an encoding use the encoding keyword.

Example:
  datafile = ffi1001(path, encoding = 'latin1')
    """
    @classmethod
    def isMine(cls, path):
        try:
            ffifmt = openf(
                path, 'r', encoding='utf-8').readline().strip()[-4:]
            if hasattr(ffifmt, 'decode'):
                ffifmt = ffifmt.decode()
            testfmt = u'1001'
            return ffifmt == testfmt
        except Exception:
            return False

    def __init__(self, path, keysubs={'/': '_'}, encoding='utf-8',
                 default_llod_flag=-8888, default_llod_value='N/A',
                 default_ulod_flag=-7777, default_ulod_value='N/A'):
        """
Arguments:
   self - implied input (not supplied in call)
   path - path to file
   keysubs - dictionary of characters to remove from variable keys and
             their replacements
   encoding - file encoding (utf-8, latin1, cp1252, etc.)
   default_llod_flag - flag value for lower limit of detections if not
                       specified
   default_llod_value - default value to use for replacement of llod_flag
   default_ulod_flag - flag value for upper limit of detections if not
                       specified
   default_ulod_value - default value to use for replacement of ulod_flag
Returns:
   out - PseudoNetCDFFile interface to data in file.
        """
        lastattr = None
        PseudoNetCDFFile.__init__(self)
        f = openf(path, 'r', encoding=encoding)
        missing = []
        units = []
        line = f.readline()
        if ',' in line:
            delim = ','
        else:
            delim = None

        def split(s):
            return [s_.strip() for s_ in s.split(delim)]

        if split(line)[-1] != '1001':
            raise TypeError(
                "File is the wrong format.  " +
                "Expected 1001; got %s" % (split(line)[-1],))

        n, self.fmt = split(line)
        # n_user_comments = 0
        n_special_comments = 0
        self.n_header_lines = int(n)
        try:
            for li in range(self.n_header_lines - 1):
                li += 2
                line = f.readline()
                LAST_VAR_DESC_LINE = 12 + len(missing)
                SPECIAL_COMMENT_COUNT_LINE = LAST_VAR_DESC_LINE + 1
                LAST_SPECIAL_COMMENT_LINE = (SPECIAL_COMMENT_COUNT_LINE +
                                             n_special_comments)
                USER_COMMENT_COUNT_LINE = (12 + len(missing) + 2 +
                                           n_special_comments)
                if li == PI_LINE:
                    self.PI_NAME = line.strip()
                elif li == ORG_LINE:
                    self.ORGANIZATION_NAME = line.strip()
                elif li == PLAT_LINE:
                    self.SOURCE_DESCRIPTION = line.strip()
                elif li == MISSION_LINE:
                    self.MISSION_NAME = line.strip()
                elif li == VOL_LINE:
                    self.VOLUME_INFO = ', '.join(split(line))
                elif li == DATE_LINE:
                    line = line.replace(',', ' ').replace(
                        '-', ' ').replace('  ', ' ').split()
                    SDATE = ", ".join(line[:3])
                    WDATE = ", ".join(line[3:])
                    self.SDATE = SDATE
                    self.WDATE = WDATE
                    self._SDATE = datetime.strptime(SDATE, '%Y, %m, %d')
                    self._WDATE = datetime.strptime(WDATE, '%Y, %m, %d')
                elif li == TIME_INT_LINE:
                    self.TIME_INTERVAL = line.strip()
                elif li == UNIT_LINE:
                    unitstr = line.replace('\n', '').replace('\r', '').strip()
                    self.INDEPENDENT_VARIABLE_DEFINITION = unitstr
                    unitstr = [s.strip() for s in unitstr.split(',')]
                    self.INDEPENDENT_VARIABLE = unitstr[0]
                    nstr = len(unitstr)
                    if nstr == 1:
                        units.append(unitstr[0])
                        self.INDEPENDENT_VARIABLE_UNITS = unitstr[0]
                    if nstr >= 2:
                        units.append(unitstr[1])
                        self.INDEPENDENT_VARIABLE_UNITS = unitstr[1]
                elif li == SCALE_LINE:
                    scales = [eval(i) for i in split(line)]
                elif li == MISSING_LINE:
                    missing = [eval(i) for i in split(line)]
                elif li > MISSING_LINE and li <= LAST_VAR_DESC_LINE:
                    nameunit = line.replace('\n', '').split(',')
                    name = nameunit[0].strip()
                    if len(nameunit) > 1:
                        units.append(nameunit[1].strip())
                    elif re.compile('(.*)\((.*)\)').match(nameunit[0]):
                        desc_groups = re.compile(
                            '(.*)\((.*)\).*').match(nameunit[0]).groups()
                        name = desc_groups[0].strip()
                        units.append(desc_groups[1].strip())
                    elif '_' in name:
                        units.append(name.split('_')[1].strip())
                    else:
                        warn('Could not find unit in string: "%s"' % line)
                        units.append(name.strip())
                elif li == SPECIAL_COMMENT_COUNT_LINE:
                    n_special_comments = int(line.replace('\n', ''))
                elif (li > SPECIAL_COMMENT_COUNT_LINE and
                      li <= LAST_SPECIAL_COMMENT_LINE):
                    colon_pos = line.find(':')
                    if (
                        li == (SPECIAL_COMMENT_COUNT_LINE + 1) and
                        colon_pos == -1
                    ):
                        k = 'SPECIAL_COMMENTS'
                        v = line.strip()
                    elif (line[:1] == ' ' or colon_pos == -1):
                        # Append to prior attribute line
                        k = lastattr
                        v = getattr(self, k, '') + line.rstrip()
                    else:
                        k = line[:colon_pos].strip().replace('/', '_')
                        v = line[colon_pos + 1:].strip()
                    setattr(self, k, v)
                    lastattr = k
                elif li == USER_COMMENT_COUNT_LINE:
                    lastattr = None
                    # n_user_comments = int(line.replace('\n', ''))
                elif (li > USER_COMMENT_COUNT_LINE and
                      li < self.n_header_lines):
                    colon_pos = line.find(':')
                    if line[:1] == ' ':
                        k = lastattr
                        v = getattr(self, k, '') + line
                    else:
                        k = line[:colon_pos].strip()
                        v = line[colon_pos + 1:].strip()
                    setattr(self, k, v)
                    lastattr = k
                elif li == self.n_header_lines:
                    varstr = line.replace(',', ' ').replace('  ', ' ')
                    variables = varstr.split()
                    for oc, nc in keysubs.items():
                        variables = [vn.replace(oc, nc) for vn in variables]
                    self.TFLAG = variables[0]
        except Exception as e:
            raise SyntaxError(
                "Error parsing icartt file %s: %s" % (path, repr(e)))

        missing = missing[:1] + missing
        scales = [1.] + scales

        nvars = len(variables)
        if hasattr(self, 'LLOD_FLAG'):
            llod_values = loddelim.sub('\n', self.LLOD_VALUE).split()
            if len(llod_values) == 1:
                llod_values *= nvars
            elif len(llod_values) == (nvars - 1):
                llod_values = ['N/A'] + llod_values
            else:
                warn(
                    'Expected 1 or %d LLOD_VALUE(s); got %d' %
                    (nvars - 1, len(llod_values))
                )
                llod_values = ['N/A'] * nvars

            assert len(llod_values) == len(variables)
            llod_values = [get_lodval(llod_val) for llod_val in llod_values]

            llod_flags = len(llod_values) * [self.LLOD_FLAG]
            llod_flags = [get_lodval(llod_flag) for llod_flag in llod_flags]
        else:
            llod_flags = [default_llod_flag] * len(scales)
            llod_values = [default_llod_value] * len(scales)

        if hasattr(self, 'ULOD_FLAG'):
            ulod_values = loddelim.sub('\n', self.ULOD_VALUE).split()
            if len(ulod_values) == 1:
                ulod_values *= nvars
            elif len(ulod_values) == (nvars - 1):
                ulod_values = ['N/A'] + ulod_values
            else:
                warn(
                    'Expected 1 or %d ULOD_VALUE(s); got %d' %
                    (nvars - 1, len(ulod_values))
                )
                ulod_values = ['N/A'] * nvars

            assert len(ulod_values) == len(variables)
            ulod_values = [get_lodval(ulod_val) for ulod_val in ulod_values]

            ulod_flags = len(ulod_values) * [self.ULOD_FLAG]
            ulod_flags = [get_lodval(ulod_flag) for ulod_flag in ulod_flags]
        else:
            ulod_flags = [default_ulod_flag] * len(scales)
            ulod_values = [default_ulod_value] * len(scales)

        data = f.read()
        datalines = data.split('\n')
        ndatalines = len(datalines)
        while datalines[-1] in ('', ' ', '\r'):
            ndatalines -= 1
            datalines.pop(-1)

        data = genfromtxt(StringIO('\n'.join(datalines).encode()),
                          delimiter=delim, dtype='d')
        data = data.reshape(ndatalines, len(variables))
        data = data.swapaxes(0, 1)
        self.createDimension('POINTS', ndatalines)
        for vi, var in enumerate(variables):
            scale = scales[vi]
            miss = missing[vi]
            unit = units[vi]
            dat = data[vi]
            llod_flag = llod_flags[vi]
            llod_val = llod_values[vi]
            ulod_flag = ulod_flags[vi]
            ulod_val = ulod_values[vi]
            vals = MaskedArray(dat * scale, mask=(dat == miss),
                               fill_value=miss)
            scale = scales[vi] = 1  # Set to 1 after applying
            tmpvar = self.variables[var] = PseudoNetCDFVariable(
                self, var, 'd', ('POINTS',), values=vals)
            tmpvar.units = unit
            tmpvar.standard_name = var
            tmpvar.missing_value = miss
            tmpvar.fill_value = miss
            tmpvar.scale = scale

            if hasattr(self, 'LLOD_FLAG'):
                tmpvar.llod_flag = llod_flag
                tmpvar.llod_value = llod_val

            if hasattr(self, 'ULOD_FLAG'):
                tmpvar.ulod_flag = ulod_flag
                tmpvar.ulod_value = ulod_val

        def dtime(s):
            return timedelta(seconds=int(s),
                             microseconds=(s - int(s)) * 1.E6)
        vtime = vectorize(dtime)
        tvar = self.variables[self.TFLAG]
        self._date_objs = (self._SDATE +
                           vtime(tvar).view(type=ndarray))


def ncf2ffi1001(f, outpath, mode='w', delim=', '):
    """
Arguments:
  f       - input file with 1-D variables and meta-data
  outpath - location to create output
  mode    - method for opening output file
  delim   - delimiter for data in output file
Returns:
  out     - output file (still open)
    """
    outfile = open(outpath, mode)
    check_for_attrs = ['PI_NAME', 'ORGANIZATION_NAME',
                       'SOURCE_DESCRIPTION', 'MISSION_NAME', 'VOLUME_INFO']
    missing_attrs = [k for k in check_for_attrs if k not in f.ncattrs()]
    if len(missing_attrs) > 0:
        warn('Missing import attributes filling with "Unknown": ' +
             ';'.join(missing_attrs))
    # header_keys = ("PI_CONTACT_INFO PLATFORM LOCATION ASSOCIATED_DATA " +
    #                "INSTRUMENT_INFO DATA_INFO UNCERTAINTY ULOD_FLAG " +
    #                "ULOD_VALUE LLOD_FLAG LLOD_VALUE DM_CONTACT_INFO " +
    #                "PROJECT_INFO STIPULATIONS_ON_USE OTHER_COMMENTS " +
    #                "REVISION").split()
    IGNORE_ATTRS = ['fmt', 'n_header_lines', 'PI_NAME', 'ORGANIZATION_NAME',
                    'SOURCE_DESCRIPTION', 'MISSION_NAME', 'VOLUME_INFO',
                    'SDATE', 'WDATE', 'TIME_INTERVAL', 'INDEPENDENT_VARIABLE',
                    'TFLAG']
    depvarkeys = [k for k in f.variables.keys() if k != f.INDEPENDENT_VARIABLE]
    myattrs = [k for k in f.ncattrs() if k not in IGNORE_ATTRS]
    print('%d, %d' % (len(myattrs) + len(depvarkeys) + 15, 1001), file=outfile)
    print(getattr(f, 'PI_NAME', 'Unknown'), file=outfile)
    print(getattr(f, 'ORGANIZATION_NAME', 'Unknown'), file=outfile)
    print(getattr(f, 'SOURCE_DESCRIPTION', 'Unknown'), file=outfile)
    print(getattr(f, 'MISSION_NAME', 'Unknown'), file=outfile)
    print(getattr(f, 'VOLUME_INFO', '1, 1'), file=outfile)
    print(f.SDATE, getattr(f, 'WDATE',
                           datetime.today().strftime('%Y, %m, %d')),
          file=outfile)
    print(getattr(f, 'TIME_INTERVAL', 0), file=outfile)
    print(f.INDEPENDENT_VARIABLE, file=outfile)
    print('%d' % len(depvarkeys), file=outfile)
    print(delim.join(['1' for k in depvarkeys]), file=outfile)
    print(delim.join([str(getattr(f.variables[k], 'missing_value', -999))
                      for k in depvarkeys]), file=outfile)
    for key, var in f.variables.items():
        if key == f.INDEPENDENT_VARIABLE:
            continue
        print(delim.join(
            [key, getattr(var, 'units', 'unknown')]), file=outfile)

    print(0, file=outfile)
    print(len(myattrs), file=outfile)
    for key in myattrs:
        print('%s: %s' % (key, getattr(f, key, '')), file=outfile)

    vals = [filled(f.variables[f.INDEPENDENT_VARIABLE][:]).ravel()]
    keys = [f.INDEPENDENT_VARIABLE]
    for key, var in f.variables.items():
        if key == f.INDEPENDENT_VARIABLE:
            continue
        keys.append(key)
        vals.append(filled(var[:]).ravel())

    print(delim.join(keys), file=outfile)
    for row in array(vals).T:
        row.tofile(outfile, format='%.6e', sep=delim)
        print('', file=outfile)

    return outfile


registerwriter('ffi1001', ncf2ffi1001)


class TestFfi1001(unittest.TestCase):
    def setUp(self):
        from PseudoNetCDF.testcase import icarttfiles_paths
        self.ffi1001path = icarttfiles_paths['ffi1001']

    def testNCF2FFI1001(self):
        import os
        ffi1001file = ffi1001(self.ffi1001path)
        outpath = self.ffi1001path + '.check'
        ncf2ffi1001(ffi1001file, outpath)
        newfile = ffi1001(outpath)
        for k, v in ffi1001file.variables.items():
            assert(k in newfile.variables)
            nv = newfile.variables[k]
            np.testing.assert_allclose(v[:], nv[:])

        os.remove(outpath)
